import sys
import qt
import os
import numpy as np
import __main__
from PythonQt import Qt
from PythonQt import QtCore
from scripts.SILA3D import GeneralModuleLogic

class SILA3DModule:
  def __init__(self, name, title, isBuiltIn, subModules):
    self.name = name
    self.nameLower = name.lower()
    self.isBuiltIn = isBuiltIn
    self.title = title
    self.subModules = subModules

results_modules = {"segmenteditor": "Editor",
                   "segmentations":"Label Maps",
                   "volumes": "Image management", 
                   "models":"3D Meshes", 
                   "labelstatistics":"Statistics"}
main_modules_ = [SILA3DModule('LoginModule', 'Login', False, {}),
                 SILA3DModule('DataModule', 'Data initialization', False, {}),
                 SILA3DModule('SegModule', 'Segmentation', False, {}),
                 SILA3DModule('ModelModule', 'Model management', False, {}),
                 SILA3DModule('Results', 'Results', True, results_modules),
                 #SILA3DModule('CoreModule', 'Segmentation Cloud', False, {})
                ]

class myMainWindow(Qt.QMainWindow):
  def __init__(self, parent=None):
    global wIcon
    super(myMainWindow, self).__init__(parent)
    self.objectName = "qSlicerMainWindow"
    self.mainWidget = myMainWidget(self) 
    self.mainWidget.objectName = "customMainWidget"
    #self.mainWidget.setWindowTitle("SILA3D Client App")
    self.setWindowTitle("SILA3D Client App")
    if wIcon is not None:
      self.setWindowIcon(wIcon)
    self.setCentralWidget(self.mainWidget)
    
  def closeEvent(self, event):
    print("closeEvent is fired")
    close = Qt.QMessageBox.question(self,
        "QUIT",
        "You are about to exit, any unsaved changes will be lost.\nDo you want to proceed?",
        Qt.QMessageBox.Yes | Qt.QMessageBox.No)
    
    if close == Qt.QMessageBox.Yes:
        print("closeEvent is accepted")
        global logicForAllModules
        logicForAllModules.exitProc(True)
        event.accept()
    else:
        print("closeEvent is cancelled")
        event.ignore()

class myMainWidget(Qt.QWidget):
  def __init__(self, parent):
    super(myMainWidget, self).__init__(parent)
    
    self.hlayout = qt.QHBoxLayout() 
    self.hlayout.objectName = "qHBoxLayout"
    self.setLayout(self.hlayout)
    
    # splitter between "layout" and "left frame"
    self.splitter = qt.QSplitter()
    self.splitter.objectName = "qSplitter"
    self.splitter.orientation = qt.Qt.Horizontal
    self.hlayout.addWidget(self.splitter)
    
    # Left frame: including control buttons and tab widget
    self.leftFrame = qt.QFrame()
    self.leftHlayout = qt.QHBoxLayout()
    self.leftFrame.setLayout(self.leftHlayout)
    self.splitter.addWidget(self.leftFrame)
    
    # main views layout
    self.layoutWidget = slicer.qMRMLLayoutWidget()
    self.layoutManager = slicer.qSlicerLayoutManager() 
    self.layoutManager.setMRMLScene(slicer.mrmlScene) 
    self.layoutManager.setScriptedDisplayableManagerDirectory(slicer.app.slicerHome + "/bin/Python/mrmlDisplayableManager") 
    self.layoutWidget.setLayoutManager(self.layoutManager) 
    slicer.app.setLayoutManager(self.layoutManager)
    self.layoutWidget.setLayout(slicer.vtkMRMLLayoutNode.SlicerLayoutConventionalView)
    self.splitter.addWidget(self.layoutWidget)
    
    # Horizontal layout: Load Data, Save Data, Module Selector
    self.vlayout = qt.QVBoxLayout()
    self.leftHlayout.addLayout(self.vlayout)

    # Tab widget
    self.mainTabWidget = qt.QTabWidget()
    self.mainTabWidget.objectName = "qMainTabHolder"
    self.mainTabWidget.setAccessibleName("SILA3D_mainWidget")
    self.resultsTabWidget = qt.QTabWidget()
    self.leftHlayout.addWidget(self.mainTabWidget)
    self.mainTabWidget.tabBarClicked.connect(self.mainTabBarClicked)
    
    # Initialize
    self.buildMainTabPanel()
    
    # for module in results_modules:
    for module in self.findSILA3DModuleByName("Results").subModules:
      self.addResultModule(module)
    

  def mainTabBarClicked(self, index):
    global logicForAllModules
    global main_modules_
    print("DEBUG: index={} module={} activeTab={} reload={}".format(
          index, main_modules_[index].name,
          logicForAllModules.ACTIVE_TAB,
          logicForAllModules.TABS_FORCE_RELOAD[index]))
     
    # if the click is done over the same active tab then do nothing
    if (logicForAllModules.ACTIVE_TAB.lower() == main_modules_[index].name.lower() and 
        logicForAllModules.TABS_FORCE_RELOAD[index] == 0):
      print("DEBUG: Same tab so no reload")
      return
    
    logicForAllModules.ACTIVE_TAB = main_modules_[index].name
    
    if (not main_modules_[index].isBuiltIn and
        logicForAllModules.TABS_FORCE_RELOAD[index] == 1): #custom modules
      print("DEBUG: Reload module")
      logicForAllModules.TABS_FORCE_RELOAD[index] = 0
      slicer.util.reloadScriptedModule(main_modules_[index].name)

  def buildMainTabPanel(self):    
    #login
    self.scrollArea1 = qt.QScrollArea(self.mainTabWidget)
    self.scrollArea1.setWidget(getattr(slicer.modules, self.findSILA3DModuleByName("LoginModule").nameLower).widgetRepresentation())
    self.scrollArea1.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOn)
    self.scrollArea1.setWidgetResizable(True)
    self.mainTabWidget.addTab(self.scrollArea1, self.findSILA3DModuleByName("LoginModule").title)
    
    #data
    self.scrollArea2 = qt.QScrollArea(self.mainTabWidget)
    self.scrollArea2.setWidget(getattr(slicer.modules, self.findSILA3DModuleByName("DataModule").nameLower).widgetRepresentation())
    self.scrollArea2.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOn)
    self.scrollArea2.setWidgetResizable(True)
    self.mainTabWidget.addTab(self.scrollArea2, self.findSILA3DModuleByName("DataModule").title)
    
    #segmentation
    self.scrollArea3 = qt.QScrollArea(self.mainTabWidget)
    self.scrollArea3.setWidget(getattr(slicer.modules, self.findSILA3DModuleByName("SegModule").nameLower).widgetRepresentation())
    self.scrollArea3.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOn)
    self.scrollArea3.setWidgetResizable(True)
    self.mainTabWidget.addTab(self.scrollArea3, self.findSILA3DModuleByName("SegModule").title)
    
    #model management
    self.scrollArea4 = qt.QScrollArea(self.mainTabWidget)
    self.scrollArea4.setWidget(getattr(slicer.modules, self.findSILA3DModuleByName("ModelModule").nameLower).widgetRepresentation())
    self.scrollArea4.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOn)
    self.scrollArea4.setWidgetResizable(True)
    self.mainTabWidget.addTab(self.scrollArea4, self.findSILA3DModuleByName("ModelModule").title)
    
    #results
    self.scrollArea5 = qt.QScrollArea(self.mainTabWidget)
    self.scrollArea5.setWidget(self.resultsTabWidget)
    self.scrollArea5.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOn)
    self.scrollArea5.setWidgetResizable(True)
    self.mainTabWidget.addTab(self.scrollArea5, self.findSILA3DModuleByName("Results").title)
    
    #old
    # scrollArea6 = qt.QScrollArea(mainTabWidget)
    # scrollArea6.setWidget(getattr(slicer.modules, findSILA3DModuleByName("CoreModule").nameLower).widgetRepresentation())
    # scrollArea6.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOn)
    # scrollArea6.setWidgetResizable(True)
    # mainTabWidget.addTab(scrollArea6, findSILA3DModuleByName("CoreModule").title)
  
  def findSILA3DModuleByName(self, name):
    global main_modules_
    r = [o for o in main_modules_ if o.nameLower == name.lower()]
    if len(r) > 0:
      return r[0]
    else:
      raise Exception("DEBUG: Module is not found!")
      
  def addResultModule(self, modulename):
    scrollArea = qt.QScrollArea(self.resultsTabWidget)
    scrollArea.setWidget(getattr(slicer.modules, modulename.lower()).widgetRepresentation())
    scrollArea.setHorizontalScrollBarPolicy(qt.Qt.ScrollBarAlwaysOn)
    scrollArea.setWidgetResizable(True)
    if (modulename.lower() in self.findSILA3DModuleByName("Results").subModules):
      self.resultsTabWidget.addTab(scrollArea, self.findSILA3DModuleByName("Results").subModules[modulename.lower()])


##################### global #####################
#
# Create a generic logic to be used in all modules
#
logicForAllModules = GeneralModuleLogic.GeneralModuleLogic()
slicer.modules.LoginModuleInstance.baseLogic = logicForAllModules
slicer.modules.DataModuleInstance.baseLogic = logicForAllModules
slicer.modules.SegModuleInstance.baseLogic = logicForAllModules
slicer.modules.ModelModuleInstance.baseLogic = logicForAllModules
# init TABS_FORCE_RELOAD based on defined modules
logicForAllModules.TABS_FORCE_RELOAD = np.zeros(len(main_modules_), dtype=int)
# store modules in the global instance
logicForAllModules.MODULES = main_modules_

#window icon
iconPath = ''
if os.path.exists('Extensions/'):
  iconPath = os.path.join('Extensions/SILA3D/Resources/Icons', 'icon.png')
if os.path.exists('scripts/'):
  iconPath = os.path.join('scripts/SILA3D/Resources/Icons', 'icon.png')
wIcon = qt.QIcon(iconPath) if iconPath != '' else None
logicForAllModules.ICONPATH = iconPath
logicForAllModules.WICON = wIcon

# main window
mainWindow = myMainWindow()
mainWindow.show()

logicForAllModules.MAIN_WINDOW = mainWindow

# another way to handle quit event
# this way it is not possible to show a confirm msg to user
# slicer.app.aboutToQuit.connect(quittingAppCleanUp)
# def quittingAppCleanUp():
  # global logicForAllModules
  # logicForAllModules.exitProc()
