# -*- coding: cp1252 -*-
import os, sys, time
import unittest
import requests
import vtk, qt, ctk, slicer, json 
#, requests
from slicer.ScriptedLoadableModule import *
import logging

#
# SILA3DModule
#

class BBox():

  def __init__(self, origX, origY, origZ, sizeX, sizeY, sizeZ):
    self.origX=origX
    self.origY=origY
    self.origZ=origZ
    self.sizeX=sizeX
    self.sizeY=sizeY
    self.sizeZ=sizeZ

    self.origX_mm=origX*0.3 - 86
    self.origY_mm=origY*0.3 - 132
    self.origZ_mm=origZ*0.3 - 32

    self.sizeX_mm=sizeX*0.3
    self.sizeY_mm=sizeY*0.3
    self.sizeZ_mm=sizeZ*0.3

    self.node = None

  def position(self):
    return (self.x,self.y,self.z)

  def setPosition(self, x, y, z):
    self.origX=x
    self.origY=y
    self.origZ=z

  def setPosition_mm(self, x, y, z):
    self.origX_mm=x
    self.origY_mm=y
    self.origZ_mm=z 

  def size(self):
    return (self.sizeX,self.sizeY,self.sizeZ)

  def setSize(self, sX, sY, sZ):
    self.sizeX=sX
    self.sizeY=sY
    self.sizeZ=sZ

  def setSize_mm(self, sX, sY, sZ):
    self.sizeX_mm=sX
    self.sizeY_mm=sY
    self.sizeZ_mm=sZ

  def resetToDefault(self):
    self.origX=0
    self.origZ=0
    self.origY=0
    self.sizeX=10
    self.sizeY=10
    self.sizeZ=10


  def compute(self, spacing, orig):
    if self.node:
      self.IJKToRAS(spacing, orig)
      nodeVolume = slicer.util.getFirstNodeByClassByName("vtkMRMLScalarVolumeNode", "B1_3_template")
      matrix = vtk.vtkMatrix4x4()
      # nodeVolume.GetIJKToRASMatrix(matrix)
      # print "ijkToRAS:"+matrix
      spacing = nodeVolume.GetSpacing()
      print "spacing:"+ str(spacing[0])+str(spacing[1])+str(spacing[2])
      
      print "orig in mm:"
      print  [self.origX_mm, self.origY_mm, self.origZ_mm]

      print "size in mm:"
      print  [self.sizeX_mm, self.sizeY_mm, self.sizeZ_mm]

      self.node.SetXYZ( [self.origX_mm, self.origY_mm, self.origZ_mm] );
      self.node.SetRadiusXYZ( self.sizeX_mm/2.0, self.sizeY_mm/2.0, self.sizeZ_mm/2.0 )

  def IJKToRAS(self, spacing, orig):
    self.origX_mm = self.origX*spacing[0] + orig[0]  # *0.3 - 86
    self.origY_mm = self.origY*spacing[1] + orig[1]  # *0.3 - 132
    self.origZ_mm = self.origZ*spacing[2] + orig[2] # *0.3 - 32

    self.sizeX_mm = self.sizeX*spacing[0]
    self.sizeY_mm = self.sizeY*spacing[1]
    self.sizeZ_mm = self.sizeZ*spacing[2]

  def RAStoIJK(self, spacing, orig):
    self.origX = round((self.origX_mm - orig[0]) / spacing[0])
    self.origY = round((self.origY_mm - orig[1]) / spacing[1])
    self.origZ = round((self.origZ_mm - orig[2]) / spacing[2])
    print 'ori : ', orig[0]  , orig[1]  , orig[2] 
    print 'space : ', spacing[0]  , spacing[1]  , spacing[2]
    
    self.sizeX = round(self.sizeX_mm/spacing[0])
    self.sizeY = round(self.sizeY_mm/spacing[1])
    self.sizeZ = round(self.sizeZ_mm/spacing[2])
    print self.origX,  self.sizeX ,self.origY, self.sizeY

  def resetTodefault(self):
    self.node.setPosition(0,0,0) 
    self.node.setSize(10,10,10)

class CoreModule(ScriptedLoadableModule):
  """Uses ScriptedLoadableModule base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def __init__(self, parent):
    ScriptedLoadableModule.__init__(self, parent)
    self.parent.title = "SILA3D" # TODO make this more human readable by adding spaces
    self.parent.categories = ["ILIAD3"]
    self.parent.dependencies = []
    self.parent.contributors = ["Barthelemy Serres (ILIAD3, Universit de Tours)", "Gaetan Galisot(LIFAT, Universit de Tours)"] # replace with "Firstname Lastname (Organization)"
    self.parent.helpText = """
This is the SILA3D python scripted module for slicer3D.
It performs a step by step segmentation on the input volume using an atlas, through a web service.
"""
    self.parent.helpText += self.getDefaultModuleDocumentationLink()
    self.parent.acknowledgementText = """
SILA3D : Segmentation Interactive using a Local Atlas
""" # replace with organization, grant and thanks.
    #self.logic = CoreModuleLogic()

#
# TemplateKeyWidget
#

class CoreModuleWidget(ScriptedLoadableModuleWidget):
  """Uses ScriptedLoadableModuleWidget base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setup(self):
    ScriptedLoadableModuleWidget.setup(self)
    self.logic = CoreModuleLogic()

    # Instantiate and connect widgets ...
    # Server Login

    self.serverLoginCollapsibleButton = ctk.ctkCollapsibleButton()
    self.serverLoginCollapsibleButton.text = "Server Login"
    self.layout.addWidget(self.serverLoginCollapsibleButton)
    


    serverLoginPanelVLayout = qt.QVBoxLayout(self.serverLoginCollapsibleButton)
    # Layout within the dummy collapsible button
    serverLoginPanelHLayout_1 = qt.QHBoxLayout() #qt.QFormLayout(sessionManagementCollapsibleButton)
    serverLoginPanelHLayout_2 = qt.QHBoxLayout()
    serverLoginPanelHLayout_3 = qt.QHBoxLayout()

    self.serverAdrsLbl = qt.QLabel()
    self.serverAdrsLbl.text = "Server:"
    self.serverAdrsEdit = qt.QLineEdit()
    self.serverAdrsEdit.text = self.logic.SERVER_ADRS
    serverLoginPanelHLayout_1.addWidget(self.serverAdrsLbl)
    serverLoginPanelHLayout_1.addWidget(self.serverAdrsEdit)
    self.serverPortLbl = qt.QLabel()
    self.serverPortLbl.text = "Port:"
    self.serverPortEdit = qt.QLineEdit()
    self.serverPortEdit.text = self.logic.SERVER_PORT
    serverLoginPanelHLayout_1.addWidget(self.serverPortLbl)
    serverLoginPanelHLayout_1.addWidget(self.serverPortEdit)
    serverLoginPanelVLayout.addLayout(serverLoginPanelHLayout_1)
    
    # Username Field
    self.usernameLbl = qt.QLabel()
    self.usernameLbl.text = "Username:"
    self.usernameLineEdit = qt.QLineEdit("")
    self.usernameLineEdit.toolTip = "Enter your login here"
    #self.usernameLineEdit.enabled = True
    self.usernameLineEdit.setText(self.logic.USER_ID)

    serverLoginPanelHLayout_2.addWidget(self.usernameLbl)
    serverLoginPanelHLayout_2.addWidget(self.usernameLineEdit)

    # Connect Button
    #
    self.connectButton = qt.QPushButton("Connect")
    self.connectButton.toolTip = "Connect to the web service."
    #self.connectButton.enabled = True
    serverLoginPanelHLayout_2.addWidget(self.connectButton)

    # Disconnect Button
    #
    self.disconnectButton = qt.QPushButton("Logout")
    self.disconnectButton.toolTip = "Disconnect from the web service."
    self.disconnectButton.enabled = False
    serverLoginPanelHLayout_2.addWidget(self.disconnectButton)
    serverLoginPanelVLayout.addLayout(serverLoginPanelHLayout_2)

    
    


    #######################################
    # Data Management


    self.dataManagementCollapsibleButton = ctk.ctkCollapsibleButton()
    self.dataManagementCollapsibleButton.text = "Data Selection"
    self.layout.addWidget(self.dataManagementCollapsibleButton)

    vLayout = qt.QVBoxLayout(self.dataManagementCollapsibleButton)
    
    dataManagementLayout_1 = qt.QHBoxLayout()
    vLayout.addLayout(dataManagementLayout_1)

    self.dataManagementHelpTextLbl = qt.QLabel()
    self.dataManagementHelpTextLbl.text = "Select here which source image you want to process, and which model/atlas to be used."
    dataManagementLayout_1.addWidget(self.dataManagementHelpTextLbl)

    # Layout within the dummy collapsible button
    dataManagementFormLayout = qt.QHBoxLayout() # qt.QFormLayout(dataManagementCollapsibleButton)
    vLayout.addLayout(dataManagementFormLayout)

    self.modelSelectedLbl = qt.QLabel()
    self.modelSelectedLbl.text = "Atlas:"
    self.modelSelector = qt.QComboBox()
    self.modelSelector.currentIndexChanged.connect(self.onModelSelectionChange)

    self.imageSelectedLbl = qt.QLabel()
    self.imageSelectedLbl.text = "Image:"
    self.imageSelector = qt.QComboBox()
    self.imageSelector.currentIndexChanged.connect(self.onImageSelectionChange)

    dataManagementFormLayout.addWidget(self.imageSelectedLbl)
    dataManagementFormLayout.addWidget(self.imageSelector)
    dataManagementFormLayout.addWidget(self.modelSelectedLbl)
    dataManagementFormLayout.addWidget(self.modelSelector)


    #
    # Reset Data Button
    #
    #self.refreshDataListButton = qt.QPushButton("Refresh")
    #self.refreshDataListButton.toolTip = "Refresh Data List."
    #self.refreshDataListButton.enabled = True
    #dataManagementFormLayout.addWidget(self.refreshDataListButton)
    
    #
    # Run The App Button
    #
    self.initializeButton = qt.QPushButton("Run the App")
    self.initializeButton.toolTip = "Run the remote Application with provided image and atlas."
    #self.initializeButton.enabled = True
    dataManagementFormLayout.addWidget(self.initializeButton)
    


    #######################################
    # Session Management

    
    self.sessionManagementCollapsibleButton = ctk.ctkCollapsibleButton()
    self.sessionManagementCollapsibleButton.text = "Session Management"
    self.layout.addWidget(self.sessionManagementCollapsibleButton)

    sessionManagementPanelHLayout_1 = qt.QHBoxLayout(self.sessionManagementCollapsibleButton)
    #
    # Load Data Button
    #
    self.loadDataButton = qt.QPushButton("Load Session Data")
    self.loadDataButton.toolTip = "Load previous results from last saved session"
    #self.loadDataButton.enabled = False
    sessionManagementPanelHLayout_1.addWidget(self.loadDataButton)

    #
    # Save Data Button
    #
    self.saveDataButton = qt.QPushButton("Save Session Data")
    self.saveDataButton.toolTip = "Save current results"
    #self.saveDataButton.enabled = False
    sessionManagementPanelHLayout_1.addWidget(self.saveDataButton)


    #############################################################
    # Algorithm Configuration Area
    #
    self.configurationCollapsibleButton = ctk.ctkCollapsibleButton()
    self.configurationCollapsibleButton.text = "Segmentation Algorithm Configuration"
    self.layout.addWidget(self.configurationCollapsibleButton)

    # Layout within the dummy collapsible button
    configurationFormLayout = qt.QFormLayout(self.configurationCollapsibleButton)

    #
    # input volume selector
    #
    #self.inputSelector = slicer.qMRMLNodeComboBox()
    #self.inputSelector.nodeTypes = ["vtkMRMLScalarVolumeNode"]
    #self.inputSelector.selectNodeUponCreation = True
    #self.inputSelector.addEnabled = False
    #self.inputSelector.removeEnabled = False
    #self.inputSelector.noneEnabled = False
    #self.inputSelector.showHidden = False
    #self.inputSelector.showChildNodeTypes = False
    #self.inputSelector.setMRMLScene( slicer.mrmlScene )
    #self.inputSelector.setToolTip( "Pick the input to the algorithm." )
    #configurationFormLayout.addRow("Input Volume: ", self.inputSelector)


    #req = requests.get("http://10.108.99.45:3000/neurobrainweb/images/")
    #if (req.status_code == "200"):
    #  JSON_obj = req.json()

    #  for img in JSON_obj:
    #    self.modelSelector.addItems(img["name"])
    
    #configurationFormLayout.addRow(self.modelSelector)

    #
    # output volume selector
    #
    #self.outputSelector = slicer.qMRMLNodeComboBox()
    #self.outputSelector.nodeTypes = ["vtkMRMLScalarVolumeNode"]
    #self.outputSelector.selectNodeUponCreation = True
    #self.outputSelector.addEnabled = True
    #self.outputSelector.removeEnabled = True
    #self.outputSelector.noneEnabled = True
    #self.outputSelector.showHidden = False
    #self.outputSelector.showChildNodeTypes = False
    #self.outputSelector.setMRMLScene( slicer.mrmlScene )
    #self.outputSelector.setToolTip( "Pick the output to the algorithm." )
    #configurationFormLayout.addRow("Output Volume: ", self.outputSelector)

    #
    # threshold value
    #
    #self.imageThresholdSliderWidget = ctk.ctkSliderWidget()
    #self.imageThresholdSliderWidget.singleStep = 0.1
    #self.imageThresholdSliderWidget.minimum = -100
    #self.imageThresholdSliderWidget.maximum = 100
    #self.imageThresholdSliderWidget.value = 0.5
    #self.imageThresholdSliderWidget.setToolTip("Set threshold value for computing the output image. Voxels that have intensities lower than this value will set to zero.")
    #configurationFormLayout.addRow("Image threshold", self.imageThresholdSliderWidget)

    #
    # check box to trigger taking screen shots for later use in tutorials
    #
    #self.enableScreenshotsFlagCheckBox = qt.QCheckBox()
    #self.enableScreenshotsFlagCheckBox.checked = 0
    #self.enableScreenshotsFlagCheckBox.setToolTip("If checked, take screen shots for tutorials. Use Save Data to write them to disk.")
    #configurationFormLayout.addRow("Enable Screenshots", self.enableScreenshotsFlagCheckBox)


    self.regionSelector = qt.QComboBox()
    self.regionSelector.currentIndexChanged.connect(self.onRegionSelectionChange)
    
    configurationFormLayout.addRow("Atlas Region: ", self.regionSelector)
    # http://10.108.99.45:3000/neurobrainweb/user/0/image/

    self.ROIInitButton = qt.QPushButton("Initialize ROI")
    self.ROIInitButton.toolTip = "ROI initialization"
    #self.ROIInitButton.enabled = True
    configurationFormLayout.addRow("ROI Position setup: ", self.ROIInitButton)

    #
    # Apply Button
    #
    self.applyButton = qt.QPushButton("Run...")
    self.applyButton.toolTip = "Run the segmentation algorithm."
    #self.applyButton.enabled = True
    configurationFormLayout.addRow("Start Segmentation", self.applyButton)


    #######################################
    # Results Management

    self.resultsDataManagementCollapsibleButton = ctk.ctkCollapsibleButton()
    self.resultsDataManagementCollapsibleButton.text = "Results Data Management"
    self.layout.addWidget(self.resultsDataManagementCollapsibleButton)

    # Layout within the dummy collapsible button
    resultDataManagementFormLayout = qt.QFormLayout(self.resultsDataManagementCollapsibleButton)

    self.segmentedRegionSelector = qt.QComboBox()
    self.segmentedRegionSelector.currentIndexChanged.connect(self.onSegmentedRegionSelectionChange)
    resultDataManagementFormLayout.addRow("Segmented regions: ",self.segmentedRegionSelector)
    # 
    # Remove region Button
    #
    self.removeRegionButton = qt.QPushButton("Remove")
    self.removeRegionButton.toolTip = "remove segmentation for selected region"
    #self.removeRegionButton.enabled = True
    resultDataManagementFormLayout.addRow(self.removeRegionButton)



    #
    # Download Data Button
    #
    self.dlSegVolDataButton = qt.QPushButton("Update")
    self.dlSegVolDataButton.toolTip = "Download results (volume)"
    #self.dlSegVolDataButton.enabled = True
    resultDataManagementFormLayout.addRow("Update Segmentation (volume): ",self.dlSegVolDataButton)

    #self.dlSegMeshDataButton = qt.QPushButton("Update")
    #self.dlSegMeshDataButton.toolTip = "Download results (meshes)"
    #self.dlSegMeshDataButton.enabled = True
    #resultDataManagementFormLayout.addRow("Update Segmentation (meshes): ",self.dlSegMeshDataButton)

    # connections
    self.connectButton.connect('clicked(bool)', self.onConnect)
    self.disconnectButton.connect('clicked(bool)', self.onDisconnect)

    #self.refreshDataListButton.connect('clicked(bool)', self.onRefreshDataList)
    self.initializeButton.connect('clicked(bool)', self.onInit)

    self.ROIInitButton.connect('clicked(bool)', self.onSetupROI)
    self.applyButton.connect('clicked(bool)', self.onApplyButton)
    #self.inputSelector.connect("currentNodeChanged(vtkMRMLNode*)", self.onSelect)
    #self.outputSelector.connect("currentNodeChanged(vtkMRMLNode*)", self.onSelect)

    self.saveDataButton.connect('clicked(bool)', self.onSaveData)
    self.loadDataButton.connect('clicked(bool)', self.onLoadData)
    self.removeRegionButton.connect('clicked(bool)', self.onRemoveRegion)
    self.dlSegVolDataButton.connect('clicked(bool)', self.onDownloadSegVolResults)
    #self.dlSegMeshDataButton.connect('clicked(bool)', self.onDownloadSegMeshResults)


    # Add vertical spacer
    self.layout.addStretch(1)

    # Refresh Apply button state
    #self.onSelect()

    self.resetUI()


  def cleanup(self):
    pass


  def delayDisplay(self,message,msec=100):
    """This utility method displays a small dialog and waits.
    This does two things: 1) it lets the event loop catch up
    to the state of the test so that rendering and widget updates
    have all taken place before the test continues and 2) it
    shows the user/developer/tester the state of the test
    so that we'll know when it breaks.
    """
    print(message)
    self.info = qt.QDialog()
    self.infoLayout = qt.QVBoxLayout()
    self.info.setLayout(self.infoLayout)
    self.label = qt.QLabel(message,self.info)
    self.infoLayout.addWidget(self.label)
    qt.QTimer.singleShot(msec, self.info.close)
    self.info.exec_()

    

  def onModelSelectionChange(self, i):
    print "DEBUG:CURRENT_MODEL:"+self.logic.CURRENT_MODEL
    self.logic.CURRENT_MODEL = self.modelSelector.currentText
    print self.logic.CURRENT_MODEL

  def onImageSelectionChange(self, i):
    print "DEBUG:CURRENT_T1:"+self.logic.CURRENT_T1
    self.logic.CURRENT_T1 = self.imageSelector.currentText
    print self.logic.CURRENT_T1

  def onSelect(self):
    self.applyButton.enabled = True #self.inputSelector.currentNode() and self.outputSelector.currentNode()

  def onApplyButton(self):
    #get bounding box position from current position
    
    self.logic.setupBoundingBox()
    if (slicer.util.confirmOkCancelDisplay("Are you sure to start segmentation process for structure named \""+str(self.logic.listRegions()[self.logic.CURRENT_REGION_ID])+"\"", windowTitle="Confirm starting segmentation on server")):
     # if sucessful segmentation
     if (self.logic.startSegmentation()):
      # Update Segmented Regions lists (populate CB with segmented Atlas Regions)
      self.updateSegmentedRegions()

      # Download last results from server
      self.logic.downloadLabelmapVolumeData()

      self.resultsDataManagementCollapsibleButton.enabled=True
      self.resultsDataManagementCollapsibleButton.collapsed=False

    #logic.test()

  def refreshDataList(self):
    
    self.modelSelector.clear()
    self.imageSelector.clear()

    imgList = self.logic.getImageList()
    for img in imgList:
      self.imageSelector.addItem(img)

    modelList = self.logic.getModelsList()
    for mdl in modelList:
      self.modelSelector.addItem(mdl)



    #enableScreenshotsFlag = self.enableScreenshotsFlagCheckBox.checked
    #imageThreshold = self.imageThresholdSliderWidget.value
    #logic.run(self.inputSelector.currentNode(), self.outputSelector.currentNode(), imageThreshold, enableScreenshotsFlag)

  def onInit(self):
    print "DEBUG:onInit("+self.logic.USER_ID+","+self.logic.CURRENT_MODEL+","+self.logic.CURRENT_T1+")"
    
    self.delayDisplay("Initializing App. Please wait...")
    
    self.dataManagementCollapsibleButton.enabled = False

    self.logic.APP_RUNNING = True

    self.logic.initApp(self.logic.USER_ID, self.logic.CURRENT_MODEL, self.logic.CURRENT_T1)
    print("DEBUG: done initApp")
    
    self.logic.initSegmentationMethod(self.logic.CURRENT_METHOD)
    print("DEBUG: done initSegmentationMethod")
      
    # Update Display
    self.logic.updateMainSceneDisplay("T1")

    # Setup Available regions (populate CB with Atlas Regions)
    self.setupAvailableRegions()

    # Load Current User Session Data
    self.onLoadData()

    # Update Segmented Regions lists (populate CB with segmented Atlas Regions)
    self.updateSegmentedRegions()

    self.sessionManagementCollapsibleButton.enabled = True
    self.sessionManagementCollapsibleButton.collapsed = False

    self.configurationCollapsibleButton.enabled = True
    self.configurationCollapsibleButton.collapsed = False

    self.resultsDataManagementCollapsibleButton.enabled = True
    self.resultsDataManagementCollapsibleButton.collapsed = False

    # Download last results from server
    self.logic.downloadLabelmapVolumeData()
    print("DEBUG: done downloadLabelmapVolumeData")
    
    print("DEBUG: exit initApp")
      

  def onRegionSelectionChange(self, i):
    #print("Selected Region ID cb index: "+str(i))
    self.logic.CURRENT_REGION_ID=self.regionSelector.itemData(i)
    print("DEBUG:Selected Region ID: "+str(self.logic.CURRENT_REGION_ID))
    self.logic.setupRegion(self.logic.CURRENT_REGION_ID)

  def onSegmentedRegionSelectionChange(self, i):
    #print("Selected Region ID cb index: "+str(i))
    self.logic.SEGMENTED_REGION_ID=self.segmentedRegionSelector.itemData(i)
    print("DEBUG:Selected Segmented Region ID: "+str(self.logic.SEGMENTED_REGION_ID))


  def setupAvailableRegions(self):
    regions = self.logic.listRegions()
    self.regionSelector.clear()
    
    for regionID in regions.keys():
      self.regionSelector.addItem(str(regionID)+"_"+regions[regionID], regionID)

  def updateSegmentedRegions(self):
    regions = self.logic.listSegmentedRegions()
    self.segmentedRegionSelector.clear()

    for regionID in regions.keys():
      self.segmentedRegionSelector.addItem(regions[regionID], regionID)

  def onSetupROI(self):
    print "DEBUG:onSetupROI"
    self.logic.setupROI(self.logic.CURRENT_REGION_ID)
    

  def onConnect(self):
    print "DEBUG:onConnectButton!"
    #initiate the values based on the input given by the user
    self.logic.USER_ID = self.usernameLineEdit.text
    self.logic.SERVER_ADRS = self.serverAdrsEdit.text
    self.logic.SERVER_PORT = self.serverPortEdit.text
    if self.logic.checkConnection():
      
      self.connectButton.enabled = False
      self.disconnectButton.enabled = True
    #dataManagementCollapsible
      self.dataManagementCollapsibleButton.enabled = True
      self.dataManagementCollapsibleButton.collapsed = False

      self.logic.LOGGEDIN = True
      self.refreshDataList();
      self.delayDisplay("Connected!")

    #logic.test()
    
    #logic = CoreModuleConnectionLogic()
    #enableScreenshotsFlag = self.enableScreenshotsFlagCheckBox.checked
    #logic.
    #imageThreshold = self.imageThresholdSliderWidget.value
    #logic.run(self.inputSelector.currentNode(), self.outputSelector.currentNode(), imageThreshold, enableScreenshotsFlag)

  def onDisconnect(self):
    print "DEBUG:onDisconnect"
    if(slicer.util.confirmOkCancelDisplay("Are you sure to logout? Any non-saved changes will be lost.", windowTitle="Confirm logout from server")):
      self.disconnectButton.enabled = False
      self.connectButton.enabled = True
    

      self.resetUI()
      
      print("APP_RUNNING: " + str(self.logic.APP_RUNNING))
      if self.logic.APP_RUNNING:
        self.onExit()

      self.logic.LOGGEDIN = False
  

  def resetUI(self):
    # connection tab
    self.serverLoginCollapsibleButton.enabled=True
    self.serverLoginCollapsibleButton.collapsed=False
    # session management
    self.sessionManagementCollapsibleButton.enabled=False
    self.sessionManagementCollapsibleButton.collapsed=True
    # dataManagement tab
    self.dataManagementCollapsibleButton.enabled=False
    self.dataManagementCollapsibleButton.collapsed=True
    # Algo configuration tab
    self.configurationCollapsibleButton.enabled=False
    self.configurationCollapsibleButton.collapsed=True
    # results display and handling
    self.resultsDataManagementCollapsibleButton.enabled=False
    self.resultsDataManagementCollapsibleButton.collapsed=True
    

  def onSaveData(self):
    print "DEBUG:onSaveData"
    if self.logic.saveRemoteSessionData():
      self.delayDisplay("Saving Session ...")
    else:
      #self.delayDisplay("Error: Session Not Saved!")
      slicer.util.errorDisplay("Error: Session Not Saved!")

  def onLoadData(self):
    print "DEBUG:onLoadData"
    if self.logic.loadRemoteSessionData():
      self.delayDisplay("Loading Session...")
    else:
      slicer.util.errorDisplay("Error: Session Not Loaded!")

  def onRemoveRegion(self):
    print "DEBUG:onRemoveRegion"
    print "DEBUG:region to be removed: "+str(self.logic.SEGMENTED_REGION_ID)
    if self.logic.removeSegmentedRegion(self.logic.SEGMENTED_REGION_ID):
      self.delayDisplay("Removing region...")
    else:
      slicer.util.errorDisplay("Error: Region Not Removed!")      

  def onDownloadSegVolResults(self):
    print "DEBUG:onDownloadResults"
    self.logic.downloadLabelmapVolumeData()

  def onDownloadSegMeshResults(self):
    print "DEBUG:onDownloadResults"
    self.logic.downloadMeshData()

  def onDataDownloaded(self):
    print "DEBUG:onDataDownloaded"

  def onExit(self):
    slicer.mrmlScene.Clear()

    self.logic.exit()
    self.logic.APP_RUNNING = False
#
# TestModuleLogic
#

class CoreModuleLogic(ScriptedLoadableModuleLogic):
  """This class should implement all the actual
  computation done by your module.  The interface
  should be such that other python code can import
  this class and make use of the functionality without
  requiring an instance of the Widget.
  Uses ScriptedLoadableModuleLogic base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """
  def __init__(self):
    self.SERVER_ADRS="http://10.108.99.45"
    self.SERVER_PORT="3000" 
    self.USER_ID="0"
    self.CURRENT_T1=""
    self.CURRENT_MODEL=""
    self.CURRENT_REGION_ID=-1
    self.CURRENT_METHOD="Local"
    self.CURRENT_ROI=BBox(0,0,0, 10, 10, 10)
    self.segmentationVolumeNode=None
    self.T1_VolumeNode=None
    self.labelmapVolumeNode=None
    self.APP_RUNNING=False
    self.LOGGEDIN=False

  def checkConnection(self):
    connected = False
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/users/"
    try:
      # print req
      ret = requests.get(req, timeout=2)
    
      if ret.status_code == 200:
        connected = True
    except :
      print("Error occured: ", sys.exc_info()[0])

    return connected


  def createColorTable(self):
    invertedocean = slicer.vtkMRMLColorTableNode()
    invertedocean.SetTypeToUser()
    invertedocean.SetNumberOfColors(256)
    invertedocean.SetName("MyInvertedOceanLUT")

    for i in range(0,255):
      invertedocean.SetColor(i, 0.0, 1 - (i+1e-16)/255.0, 1.0, 1.0)

    slicer.mrmlScene.AddNode(invertedocean)

  def hasImageData(self,volumeNode):
    """This is an example logic method that
    returns true if the passed in volume
    node has valid image data
    """
    if not volumeNode:
      logging.debug('hasImageData failed: no volume node')
      return False
    if volumeNode.GetImageData() is None:
      logging.debug('hasImageData failed: no image data in volume node')
      return False
    return True

  def isValidInputOutputData(self, inputVolumeNode, outputVolumeNode):
    """Validates if the output is not the same as input
    """
    if not inputVolumeNode:
      logging.debug('isValidInputOutputData failed: no input volume node defined')
      return False
    if not outputVolumeNode:
      logging.debug('isValidInputOutputData failed: no output volume node defined')
      return False
    if inputVolumeNode.GetID()==outputVolumeNode.GetID():
      logging.debug('isValidInputOutputData failed: input and output volume is the same. Create a new volume for output to avoid this error.')
      return False
    return True

  def takeScreenshot(self,name,description,type=-1):
    # show the message even if not taking a screen shot
    slicer.util.delayDisplay('Take screenshot: '+description+'.\nResult is available in the Annotations module.', 3000)

    lm = slicer.app.layoutManager()
    # switch on the type to get the requested window
    widget = 0
    if type == slicer.qMRMLScreenShotDialog.FullLayout:
      # full layout
      widget = lm.viewport()
    elif type == slicer.qMRMLScreenShotDialog.ThreeD:
      # just the 3D window
      widget = lm.threeDWidget(0).threeDView()
    elif type == slicer.qMRMLScreenShotDialog.Red:
      # red slice window
      widget = lm.sliceWidget("Red")
    elif type == slicer.qMRMLScreenShotDialog.Yellow:
      # yellow slice window
      widget = lm.sliceWidget("Yellow")
    elif type == slicer.qMRMLScreenShotDialog.Green:
      # green slice window
      widget = lm.sliceWidget("Green")
    else:
      # default to using the full window
      widget = slicer.util.mainWindow()
      # reset the type so that the node is set correctly
      type = slicer.qMRMLScreenShotDialog.FullLayout

    # grab and convert to vtk image data
    qimage = ctk.ctkWidgetsUtils.grabWidget(widget)
    imageData = vtk.vtkImageData()
    slicer.qMRMLUtils().qImageToVtkImageData(qimage,imageData)

    annotationLogic = slicer.modules.annotations.logic()
    annotationLogic.CreateSnapShot(name, description, type, 1, imageData)

  def initApp(self, username, model, t1):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/init/?mode=Seg&model="+model+"&t1="+t1
    print req
    ret = requests.get(req)
    self.initialized = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        self.initialized=True

    return self.initialized

  def exit(self):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/exit"
    print req
    ret = requests.get(req)
    processDone = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      #if resp["status"] == "OK":
      processDone = True
      self.delayDisplay('Session Terminated!')

      #Clear Scene
      # slicer.mrmlScene.clear()

    return processDone


  def initSegmentationMethod(self, method):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/segmethod/?m="+method
    print req
    ret = requests.get(req)
    initialized = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        initialized=True

    return initialized

  def listRegions(self):
    regionDic={}
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/regions/"
    print req
    ret = requests.get(req)
    initialized = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      #print resp
      if resp["status"] == "OK":
        #print resp["data"]["regions"]
        regions = resp["data"]["regions"]
        
        #print "regions="+regions
        for region in regions:
          regionDic[int(region["id"])] = region["name"]

      print regionDic

    return regionDic

  def listSegmentedRegions(self):
    segRegionDic={}
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/segmented/"
    print req
    ret = requests.get(req)
    initialized = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      #print resp
      if resp["status"] == "OK":
        #print resp["data"]["regions"]
        regions = resp["data"]
        
        #print "regions="+regions
        for region in regions:
          segRegionDic[int(region["id"])] = region["id"]+" "+region["name"]

      print segRegionDic

    return segRegionDic


  def removeSegmentedRegion(self, region_id):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/region/remove/?id="+str(region_id)
    print req
    ret = requests.get(req)
    regionIDremoved = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        regionIDremoved = True

    return regionIDremoved


  def loadRemoteSessionData(self):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/results/load/?modelName="+self.CURRENT_MODEL
    print req
    ret = requests.get(req)
    sessionLoaded = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        sessionLoaded = True

    return sessionLoaded

  def saveRemoteSessionData(self):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/results/save/?modelName="+self.CURRENT_MODEL
    print req
    ret = requests.get(req)
    sessionSaved = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        sessionSaved = True

    return sessionSaved


  def getCurrentImageURL(self):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/user/"+self.USER_ID+"/image/"
    print req
    ret = requests.get(req)
    imgURL = ""
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      imgURL=self.SERVER_ADRS+":"+self.SERVER_PORT+"/Data/Images/"+resp["name"]
      
    return imgURL


  def setupRegion(self, ID):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/region/?id="+str(ID)
    print req
    ret = requests.get(req)
    regionIDChanged = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        regionIDChanged = True

      slicer.util.showStatusMessage(resp["message"])

    return regionIDChanged

  def setupROI(self, roiID):

    if(self.CURRENT_ROI.node == None):
      self.CURRENT_ROI.node = slicer.vtkMRMLAnnotationROINode()
      self.CURRENT_ROI.node.Initialize(slicer.mrmlScene)

    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/bbox/auto"
    print req
    ret = requests.get(req)
    #bbox = []
    print ret
    if ret.status_code == 200:
      try:
        resp = ret.json()
        print resp
        if (resp["status"] == "OK") | (resp["status"] == "WARN"): # | (resp["Status"] == "OK"):
          # Debug
          print resp["data"]["boundingBox"]
          bbox = resp["data"]["boundingBox"]
          self.CURRENT_ROI.setPosition(int(bbox["origin"][0]), int(bbox["origin"][1]), int(bbox["origin"][2]))
          self.CURRENT_ROI.setSize(int(bbox["dim"][0]), int(bbox["dim"][1]), int(bbox["dim"][2]))

          #layoutManager = slicer.app.layoutManager()
          ##red = layoutManager.sliceWidget('Red')
          #redLogic = red.sliceLogic()
          # Print current slice offset position
          # print(redLogic.GetSliceOffset())
          # Change slice position
          #redLogic.SetSliceOffset(int(bbox["origin"][0])+int(bbox["dim"][0])/2.0)

        else:
          msgText=resp["message"]
          slicer.util.showStatusMessage(msgText)
          slicer.util.errorDisplay(msgText)
          self.CURRENT_ROI.resetToDefault()
      except:
        msgText="Error retrieving boundingbox response data from server!"
        slicer.util.errorDisplay(msgText, windowTitle=None)
        pass

    
    # T1 image is self.segmentationVol (get spacing and origin in this data structure)
    spacing = self.T1_VolumeNode.GetSpacing()
    orig = self.T1_VolumeNode.GetOrigin()
    # spacing = [0.3, 0.3, 0.3]
    # orig = [-86.0, -132.0, -32.0]

    self.CURRENT_ROI.compute(spacing, orig) # self.segmentationVol.GetSpacing)
    

    #roi.SetRadiusXYZ()
    #if roiID>0:
    #  print "ROI ID > 0"
    # switch module
    # mainWindow = slicer.util.mainWindow()
    # mainWindow.moduleSelector().selectModule('CropVolume')

  def setupBoundingBox(self):
    
    lim = [0,0,0,0,0,0]
    self.CURRENT_ROI.node.GetBounds(lim)
    self.CURRENT_ROI.setPosition_mm(lim[0],lim[2],lim[4])
    self.CURRENT_ROI.setSize_mm(lim[1]-lim[0],lim[3]-lim[2],lim[5]-lim[4])
    spacing = self.T1_VolumeNode.GetSpacing()
    orig = self.T1_VolumeNode.GetOrigin()
    self.CURRENT_ROI.RAStoIJK(spacing,orig)
    
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/bbox/?x="+str(self.CURRENT_ROI.origX)+"&y="+str(self.CURRENT_ROI.origY)+"&z="+str(self.CURRENT_ROI.origZ)+"&w="+str(self.CURRENT_ROI.sizeX)+"&h="+str(self.CURRENT_ROI.sizeY)+"&d="+str(self.CURRENT_ROI.sizeZ)
    print req
    ret = requests.get(req)
    processDone = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        processDone = True

        print "BBox updated."
        #self.delayDisplay('BBox Updated!')

    return processDone


  def startSegmentation(self):
    req = self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/start"
    print req
    ret = requests.get(req, timeout=None)
    processDone = False
    print ret
    if ret.status_code == 200:
      resp = ret.json()
      print resp
      if resp["status"] == "OK":
        processDone = True
        self.delayDisplay('Segmentation Finished!')

    return processDone


  def updateMainSceneDisplay(self, mainImg):
    imgURL=self.getCurrentImageURL()
    #resultImgURL=self.SERVER_ADRS+":"+self.SERVER_PORT+"/"+self.USER_ID+"/results/download/"
    import urllib
    downloads = (
        (imgURL,        self.CURRENT_T1,  slicer.util.loadVolume),
        )

    for url,name,loader in downloads:
      filePath = slicer.app.temporaryPath + '/' + name
      if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
        logging.info('Requesting download %s from %s...\n' % (name, url))
        urllib.urlretrieve(url, filePath)
      if loader:
        logging.info('Loading %s...' % (name,))
        print "filePath="+filePath
        [success, loadedVolumeNode] = loader(filePath, returnNode=True)
        #print success
        #hasLoaded = loader(filePath)
        self.T1_VolumeNode = loadedVolumeNode # slicer.util.getNode(pattern=name)
    
    self.delayDisplay('Finished with download and loading')

    #[success, loadedVolumeNode] = slicer.util.getNode(pattern="B2_3")

  def downloadLabelmapVolumeData(self):

    #[success, labelmapVolumeNode] = slicer.util.loadLabelVolume(filepath, returnNode=True)
    #segNode = slicer.mrmlScene.AddNewNodeByClass("vtkMRMLSegmentationNode")
    #slicer.modules.segmentations.logic().ImportLabelmapToSegmentationNode(labelmapVolumeNode, segNode)
    resultImgURL=self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/results/download/"
    import urllib
    downloads = (
          (resultImgURL,  "results.nii.gz",    slicer.util.loadLabelVolume),
        )

    for url,name,loader in downloads:
      filePath = slicer.app.temporaryPath + '/' + name
      #if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
      if os.path.exists(filePath):
        logging.info("previous file exits: deleting..")
        os.remove(filePath)
        logging.info("previous file exits: deleted.")
      
      logging.info('Requesting download %s from %s...\n' % (name, url))
      urllib.urlretrieve(url, filePath)
      if loader:
        msg='Loading %s...' % (name,)
        logging.info(msg)
        slicer.util.showStatusMessage(msg)
        #DEBUG
        print "filePath="+filePath
        #[success, loadedVolumeNode] = slicer.util.loadVolume(filePath, returnNode=True)
        #[success, labelmapVolumeNode] = slicer.util.loadVolume(filePath, returnNode=True)
        #print type(self.labelmapVolumeNode)
        #print type(self.segmentationVolumeNode)
        
        if self.segmentationVolumeNode != None:
          slicer.mrmlScene.RemoveNode(self.segmentationVolumeNode)
        
        if self.labelmapVolumeNode != None:
          slicer.mrmlScene.RemoveNode(self.labelmapVolumeNode)

        
        #if self.segmentationVolumeNode != None:
        #    slicer.mrmlScene.RemoveNode(self.segmentationVolumeNode)

        #self.segmentationVolumeNode = slicer.vtkMRMLSegmentationNode()
        self.segmentationVolumeNode = slicer.mrmlScene.AddNewNodeByClass('vtkMRMLSegmentationNode', 'SegmentationLabelmap')
        

        #print success
        #allSegmentsLabelmapNode = slicer.vtkMRMLLabelMapVolumeNode()
        #allSegmentsLabelmapNode.SetName('AllSegmentsLabelmap')
        #slicer.mrmlScene.AddNode(allSegmentsLabelmapNode)

        [success, allSegmentsLabelmapNode] = loader(filePath, returnNode=True)
        msg='%s Loaded.' % (name,)
        slicer.util.showStatusMessage(msg)

        allSegmentsLabelmapNode.SetName('AllSegmentsLabelmap')
        
        #if(success):
        self.labelmapVolumeNode = allSegmentsLabelmapNode

          # DEBUG
          # print type(allSegmentsLabelmapNode)
          # print type(self.labelmapVolumeNode)
          # print type(self.segmentationVolumeNode)
        
        #segNode = slicer.mrmlScene.AddNewNodeByClass("vtkMRMLSegmentationNode")
        slicer.vtkSlicerSegmentationsModuleLogic.ImportLabelmapToSegmentationNode(self.labelmapVolumeNode, self.segmentationVolumeNode) #, [51,70])
        
        #slicer.util.resetSliceViews()

        

        layoutManager = slicer.app.layoutManager()
        for sliceViewName in layoutManager.sliceViewNames():
          controller = layoutManager.sliceWidget(sliceViewName).sliceController()
          controller.setSliceVisible(True)

        slicer.util.resetThreeDViews()
        
        # Create 3D
        #self.segmentationVolumeNode.CreateClosedSurfaceRepresentation()
        
    
    self.delayDisplay('Finished with download and loading')


  def downloadScalarVolumeData(self):
    resultImgURL=self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/results/download/"
    import urllib
    downloads = (
          (resultImgURL,  "results.nii.gz",    slicer.util.loadVolume),
        )

    for url,name,loader in downloads:
      filePath = slicer.app.temporaryPath + '/' + name
      #if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
      if os.path.exists(filePath):
        logging.info("previous file exits: deleting..")
        os.remove(filepath)
        logging.info("previous file exits: deleted.")


      logging.info('Requesting download %s from %s...\n' % (name, url))
      urllib.urlretrieve(url, filePath)
      
      if loader:
        logging.info('Loading %s...' % (name,))
        print "filePath="+filePath
        #[success, loadedVolumeNode] = slicer.util.loadVolume(filePath, returnNode=True)
        [success, labelmapVolumeNode] = loader(filePath, returnNode=True)
        #segVolumeNode = slicer.vtkMRMLSegmentationNode()
        #print success
        #hasLoaded = loader(filePath)

        self.labelmapVolumeNode = labelmapVolumeNode
        # self.segmentationVol = slicer.util.getNode(pattern="FA")
    
    self.delayDisplay('Finished with download and loading')

  def downloadMeshData(self, regionID):
    
    resultMeshReq=self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/"+self.USER_ID+"/results/mesh/?id="+str(regionID)


    import urllib
    downloads = (
          (resultImgURL,  str(regionID)+".vtk",    slicer.util.loadMesh),
        )

    for url,name,loader in downloads:
      filePath = slicer.app.temporaryPath + '/' + name
      if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
        logging.info('Requesting download %s from %s...\n' % (name, url))
        urllib.urlretrieve(url, filePath)
      if loader:
        logging.info('Loading %s...' % (name,))
        print "filePath="+filePath
        #[success, loadedVolumeNode] = slicer.util.loadVolume(filePath, returnNode=True)
        #print success
        [success, loadedMeshNode] = loader(filePath, returnNode=True)
        self.segmentationMesh = loadedMeshNode
    
    self.delayDisplay('Finished with download and loading')


  def test(self):
    print("""<filter-start><filter-name>TestFilter</filter-name><filter-comment>ibid</filter-comment></filter-start>""")
    sys.stdout.flush()

    for i in range(0,10):
      print("""<filter-progress>{}</filter-progress>""".format(i/10.0))
      sys.stdout.flush()
      time.sleep(0.5)

    print("""<filter-end><filter-name>TestFilter</filter-name><filter-time>10</filter-time></filter-end>""")
    sys.stdout.flush()

  def getImageList(self):

    ret = requests.get(self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/images")
    imgList = []
    if ret.status_code == 200:
      objList = ret.json()
      for obj in objList:
        print ("Adding " + obj["name"])
        imgList.append(obj["name"])

    return imgList

  def getModelsList(self):

    ret = requests.get(self.SERVER_ADRS+":"+self.SERVER_PORT+"/neurobrainweb/models")
    modelList = []
    if ret.status_code == 200:
      objList = ret.json()
      for obj in objList:
        print ("Adding " + obj["name"])
        modelList.append(obj["name"])

    return modelList



  def run(self, inputVolume, outputVolume, imageThreshold, enableScreenshots=0):
    """
    Run the actual algorithm
    """

    if not self.isValidInputOutputData(inputVolume, outputVolume):
      slicer.util.errorDisplay('Input volume is the same as output volume. Choose a different output volume.')
      return False

    logging.info('Processing started')

    # Compute the thresholded output volume using the Threshold Scalar Volume CLI module
    cliParams = {'InputVolume': inputVolume.GetID(), 'OutputVolume': outputVolume.GetID(), 'ThresholdValue' : imageThreshold, 'ThresholdType' : 'Above'}
    cliNode = slicer.cli.run(slicer.modules.thresholdscalarvolume, None, cliParams, wait_for_completion=True)

    # Capture screenshot
    if enableScreenshots:
      self.takeScreenshot('TestModuleTest-Start','MyScreenshot',-1)

    logging.info('Processing completed')

    return True


class CoreModuleTest(ScriptedLoadableModuleTest):
  """
  This is the test case for your scripted module.
  Uses ScriptedLoadableModuleTest base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setUp(self):
    """ Do whatever is needed to reset the state - typically a scene clear will be enough.
    """
    slicer.mrmlScene.Clear(0)

  def runTest(self):
    """Run as few or as many tests as needed here.
    """
    self.setUp()
    self.test_TestModuleTest1()

  def test_TestModuleTest1(self):
    """ Ideally you should have several levels of tests.  At the lowest level
    tests should exercise the functionality of the logic with different inputs
    (both valid and invalid).  At higher levels your tests should emulate the
    way the user would interact with your code and confirm that it still works
    the way you intended.
    One of the most important features of the tests is that it should alert other
    developers when their changes will have an impact on the behavior of your
    module.  For example, if a developer removes a feature that you depend on,
    your test should break so they know that the feature is needed.
    """

    self.delayDisplay("Starting the test")
    #
    # first, get some data
    #
    import urllib
    downloads = (
        ('http://slicer.kitware.com/midas3/download?items=5767', 'FA.nrrd', slicer.util.loadVolume),
        )

    for url,name,loader in downloads:
      filePath = slicer.app.temporaryPath + '/' + name
      if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
        logging.info('Requesting download %s from %s...\n' % (name, url))
        urllib.urlretrieve(url, filePath)
      if loader:
        logging.info('Loading %s...' % (name,))
        loader(filePath)
    self.delayDisplay('Finished with download and loading')

    volumeNode = slicer.util.getNode(pattern="FA")
    #logic = CoreModuleTestLogic()
    self.assertIsNotNone( self.logic.hasImageData(volumeNode) )
    self.delayDisplay('Test passed!')
