import sys
import requests
import json

class GeneralModuleLogic():
  
  def __init__(self, x = None):
    self.SERVER_ADRS = "http://demo-iliad3.univ-tours.fr" #"http://127.0.0.2"
    self.SERVER_PORT = "8081" #"3000" 
    self.USER_ID = ""
    self.USER_NAME = ""
    self.USER_PASS = ""
    self.USER_ROLE = "user"
    self.LOGGEDIN = False
    
    self.CURRENT_MODEL = ""
    self.CURRENT_MODEL_FLD = ''
    self.CURRENT_MODEL_PUBLIC = False
    self.CURRENT_T1 = ""
    self.CURRENT_T1_FILE = ''
    self.CURRENT_T1_PUBLIC = False
    self.APP_RUNNING = False
    self.DATA_NEED_SAVE = False
    self.SEG_TIMER = None
    self.SEG_CNT_TIMER_RAN = 0
    self.LOADED_SESSION_DIRNAME = ""
    self.LEARN_TIMER = None
    self.LEARN_CNT_TIMER_RAN = 0
    self.REGION_DICTIONARY = {}
    self.SEGMENTED_REGION_LOADED_SESSION = []
    self.MODELS = []
    self.IMAGES = []
    self.FORCE_RETRY = False
    self.FORCE_RETRY_MODEL = False
    
    self.MODULES = []
    self.TABS_FORCE_RELOAD = []
    self.ACTIVE_TAB = "LoginModule"
    self.MAIN_WINDOW = None
    self.ICONPATH = ''
    self.WICON = None
  
  def forceModuleReload(self, module):
    if (module):
      index = self.findSILA3DModuleByName_index(module)
    else:
      index = -1

    if index < 0: #reload all
      self.TABS_FORCE_RELOAD[self.TABS_FORCE_RELOAD != 1] = 1
      # except login we dont need to reload it
      self.TABS_FORCE_RELOAD[0] = 0
    else:
      self.TABS_FORCE_RELOAD[index] = 1
  
  def checkConnection(self):
    connected = False
    connectToServer = False
    msg = ''
    headers = {'Content-Type': 'application/json', 'Accept':'application/json'}
    dt = {"id": 0, "name": self.USER_NAME, "pass": self.USER_PASS}
    req = self.SERVER_ADRS + ":" + self.SERVER_PORT + "/neurobrainweb/user/in"
    try:
      ret = requests.post(req, data = json.dumps(dt), headers = headers)
      # ret = requests.get(req, timeout = 2)
      if ret.status_code == 200:
        resp = ret.json()
        if resp["status"] == "OK" and resp['id'] is not None:
          self.USER_ID = str(resp['id'])
          self.USER_ROLE = str(resp['role'])
          connected = True
        else:
          msg = resp['message']
      connectToServer = True
    except :
      print("Error occured: ", sys.exc_info()[0])

    return [connectToServer, connected, msg]
  
  def findSILA3DModuleByName_index(self, name):
    r = [i for i, o in enumerate(self.MODULES) if o.nameLower == name.lower()]
    if len(r) > 0:
      return r[0]
    else:
      raise Exception("DEBUG: Module is not found!")
      
  def switchToTab(self, slicer, name):
    tab = None
    if self.MAIN_WINDOW == None:
      # print("getting tab from slicer")
      tab = slicer.util.findChildren(
        widget = slicer.util.findChildren(
          #widget = slicer.util.lookupTopLevelWidget('qSlicerAppMainWindow'), 
          widget = slicer.util.lookupTopLevelWidget('qSlicerMainWindow'),
          name = 'qSplitter')[0], 
        name = 'qMainTabHolder')[0]
    else:
      # print("getting tab straight by reference")
      tab = self.MAIN_WINDOW.mainWidget.mainTabWidget
    if tab != None:
      index = self.findSILA3DModuleByName_index(name)
      tab.setCurrentIndex(index)
      tab.tabBarClicked(index)
  
  def exitProc(self, logoff):
    lo = -1
    if logoff:
      lo = 1
    else:
      lo = 0
    req = (self.SERVER_ADRS + ":" + self.SERVER_PORT + 
            "/neurobrainweb/" + self.USER_ID + "/exit?logoff=" + str(lo))
    print(req)
    ret = requests.get(req)
    processDone = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        processDone = True
        # stop all timers
        if self.SEG_TIMER != None and self.SEG_TIMER.isActive():
          self.SEG_TIMER.stop()
        if self.SEG_CNT_TIMER_RAN > 0:
          self.SEG_CNT_TIMER_RAN = 0
        if self.LEARN_TIMER != None and self.LEARN_TIMER.isActive():
          self.LEARN_TIMER.stop()
        if self.LEARN_CNT_TIMER_RAN > 0:
          self.LEARN_CNT_TIMER_RAN = 0

    return processDone
  
  def resetSegModuleGlobalParams(self):
    self.CURRENT_MODEL = ""
    self.CURRENT_MODEL_FLD = ''
    self.CURRENT_MODEL_PUBLIC = False
    self.CURRENT_T1 = ""
    self.CURRENT_T1_FILE = ''
    self.CURRENT_T1_PUBLIC = False
    self.APP_RUNNING = False
    self.DATA_NEED_SAVE = False
    self.SEG_TIMER = None
    self.SEG_CNT_TIMER_RAN = 0
    self.LOADED_SESSION_DIRNAME = ""
    self.LEARN_TIMER = None
    self.LEARN_CNT_TIMER_RAN = 0
    self.REGION_DICTIONARY = {}
    self.SEGMENTED_REGION_LOADED_SESSION = []

  def hanldeFatalError(self, ret): # this is probably when CORE exits unexpectedly
    msg = ""
    retry = False
    resp = ret.json()
    if ret.status_code == 500:
      if resp["message"] == "process exited": # client should try to reinitialize exe
        msg = ("Unfortunately, a fatal error has occurred on the server.\n" +
            "Please wait while trying to reconnect and get your last data.\n" + 
            "If the error presists, please contact the support team.")
        retry = True
      elif resp["message"] == "process busy": # core is busy executing another task so request is ignored
        msg = "custom"
        retry = True
      else: # something has failed but exe is still up. normally this should not happen
        msg = "Failed to process your request, please try again!"
    else: # unknown and unexpected status code/critical
      msg = ("Unfortunately, a fatal error has occurred on the server!\n" + 
        "Pelase start over by logging off and on again, or restarting the application.\n" + 
        "If the error presists, please contact the support team.")
    return [msg, retry]

