# -*- coding: cp1252 -*-
import os, sys, time
import unittest
import requests
import vtk, qt, ctk, slicer, json
from slicer.ScriptedLoadableModule import *
import logging

#
# SILA3DModule - Login tab
#

class LoginModule(ScriptedLoadableModule):
  """Uses ScriptedLoadableModule base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def __init__(self, parent):
    ScriptedLoadableModule.__init__(self, parent)
    self.parent.title = "SILA3D" # TODO make this more human readable by adding spaces
    self.parent.categories = ["SILA3D - Login"]
    self.parent.dependencies = []
    self.parent.contributors = ["Barthelemy Serres (ILIAD3, Universit de Tours)", "Gaetan Galisot(LIFAT, Universit de Tours)"] # replace with "Firstname Lastname (Organization)"
    self.parent.helpText = """
This is the SILA3D python scripted module for slicer3D.
It performs a step by step segmentation on the input volume using an atlas, through a web service.
"""
    self.parent.helpText += self.getDefaultModuleDocumentationLink()
    self.parent.acknowledgementText = """
SILA3D : Segmentation Interactive using a Local Atlas
""" # replace with organization, grant and thanks.
    
    # M.D.
    self.baseLogic = None

#
# TemplateKeyWidget
#

class LoginModuleWidget(ScriptedLoadableModuleWidget):
  """Uses ScriptedLoadableModuleWidget base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setup(self):
    self.developerMode = False
    
    ScriptedLoadableModuleWidget.setup(self)
    self.logic = LoginModuleLogic(slicer.modules.LoginModuleInstance.baseLogic, self)
    
    # Instantiate and connect widgets ...
    # Server Login

    serverLoginPanelHLayout_1 = qt.QHBoxLayout()
    serverLoginPanelHLayout_2 = qt.QHBoxLayout()
    serverLoginPanelHLayout_3 = qt.QHBoxLayout()
    
    #IP address
    self.serverAdrsLbl = qt.QLabel()
    self.serverAdrsLbl.text = "Server: "
    self.serverAdrsLbl.setStyleSheet("""
        min-width: 40px;
    """)
    serverLoginPanelHLayout_1.addWidget(self.serverAdrsLbl)
    self.serverAdrsEdit = qt.QLineEdit()
    self.serverAdrsEdit.toolTip = "Enter the server's IP address"
    self.serverAdrsEdit.text = self.logic.baseLogic.SERVER_ADRS # def val
    #self.serverAdrsEdit.enabled = self.developerMode
    serverLoginPanelHLayout_1.addWidget(self.serverAdrsEdit)
    
    #Port
    self.serverPortLbl = qt.QLabel()
    self.serverPortLbl.text = "Port: "
    self.serverPortLbl.setStyleSheet("""
        min-width: 40px;
    """)
    serverLoginPanelHLayout_1.addWidget(self.serverPortLbl)
    self.serverPortEdit = qt.QLineEdit()
    self.serverPortEdit.toolTip = "Enter the server's port number"
    self.serverPortEdit.text = self.logic.baseLogic.SERVER_PORT # def val
    #self.serverPortEdit.enabled = self.developerMode
    serverLoginPanelHLayout_1.addWidget(self.serverPortEdit)

    #username & pass (for later)
    self.usernameLbl = qt.QLabel()
    self.usernameLbl.text = "Username: "
    self.usernameLbl.setStyleSheet("""
        min-width: 40px;
    """)
    serverLoginPanelHLayout_2.addWidget(self.usernameLbl)
    self.usernameLineEdit = qt.QLineEdit("")
    self.usernameLineEdit.toolTip = "Enter your username here"
    self.usernameLineEdit.setText(self.logic.baseLogic.USER_NAME) #def val
    serverLoginPanelHLayout_2.addWidget(self.usernameLineEdit)
    self.userpassLbl = qt.QLabel()
    self.userpassLbl.text = "Password: "
    self.userpassLbl.setStyleSheet("""
        min-width: 40px;
    """)
    serverLoginPanelHLayout_2.addWidget(self.userpassLbl)
    self.userpassLineEdit = qt.QLineEdit("")
    self.userpassLineEdit.toolTip = "Enter your password here"
    self.userpassLineEdit.setEchoMode(qt.QLineEdit.Password)
    self.userpassLineEdit.setText(self.logic.baseLogic.USER_PASS) #def val
    serverLoginPanelHLayout_2.addWidget(self.userpassLineEdit)
    
    #buttons
    self.loginButton = qt.QPushButton("Login")
    # self.loginButton.toolTip = "Login to the server"
    self.loginButton.connect('clicked(bool)', self.onConnect)
    self.loginButton.setStyleSheet("""
        min-width: 80px;
        height: 17px;
        margin-top: 15px;
    """)
    serverLoginPanelHLayout_3.addWidget(self.loginButton)
    self.logoutButton = qt.QPushButton("Logout")
    # self.logoutButton.toolTip = "Logout from the server"
    self.logoutButton.enabled = False
    self.logoutButton.connect('clicked(bool)', self.onDisconnect)
    self.logoutButton.setStyleSheet("""
        min-width: 80px;
        height: 17px;
        margin-top: 15px;
    """)
    serverLoginPanelHLayout_3.addWidget(self.logoutButton)
    
    self.layout.addLayout(serverLoginPanelHLayout_1)
    self.layout.addLayout(serverLoginPanelHLayout_2)
    self.layout.addLayout(serverLoginPanelHLayout_3)
    
    # print(slicer.util.findChildren(
      # widget = slicer.util.findChildren(
        # widget = slicer.util.lookupTopLevelWidget('qSlicerAppMainWindow'), 
        # name = 'qSplitter')[0], 
      # name = 'qMainTabHolder')[0])
    # print("here")
    # widget = None
    # for w in slicer.app.topLevelWidgets():
      # if w.name == 'customMainWindow':
        # print(w)
        # widget = w
    # print(slicer.util.lookupTopLevelWidget('customMainWindow'))
    # ww = slicer.util.findChildren(widget)
    # print("here1")
    # for w1 in ww:
      # print(w1)
    # print(dir(slicer.util))
    # print(slicer.modules.LoginModuleWidget.logic.SERVER_ADRS)
    # print(slicer.modules.loginmodule.widgetRepresentation().self().logic())
    # print(slicer.app.moduleManager().module("LoginModule").logic().IsTypeOf("GeneralModuleLogic"))
    # self.parent.layout.tabBarClicked.connect(testBarClicked)
    
    # Add vertical spacer
    self.layout.addStretch(1)
    
  def cleanup(self):
    pass


  def delayDisplay(self,message,msec=100):
    """This utility method displays a small dialog and waits.
    This does two things: 1) it lets the event loop catch up
    to the state of the test so that rendering and widget updates
    have all taken place before the test continues and 2) it
    shows the user/developer/tester the state of the test
    so that we'll know when it breaks.
    """
    print(message)
    self.info = qt.QDialog()
    self.info.setWindowTitle('Info')
    if self.logic.baseLogic.WICON is not None:
      self.info.setWindowIcon(self.logic.baseLogic.WICON)
    self.info.setWindowFlags(qt.Qt.WindowTitleHint)
    self.info.resize(150,30)
    self.infoLayout = qt.QVBoxLayout()
    self.info.setLayout(self.infoLayout)
    self.label = qt.QLabel(message,self.info)
    self.infoLayout.addWidget(self.label, qt.Qt.AlignCenter)
    qt.QTimer.singleShot(msec, self.info.close)
    self.info.exec_()

  def onConnect(self):
    if (self.serverAdrsEdit.text == '' or self.serverPortEdit.text == '' or
        self.usernameLineEdit.text == '' or self.userpassLineEdit.text == ''):
        self.delayDisplay('Please fill in you credentials to login!', 2000)
        return
    print("DEBUG: onConnectButton!")
    #initiate the values based on the input given by the user
    self.logic.baseLogic.SERVER_ADRS = self.serverAdrsEdit.text
    self.logic.baseLogic.SERVER_PORT = self.serverPortEdit.text
    self.logic.baseLogic.USER_NAME = self.usernameLineEdit.text
    self.logic.baseLogic.USER_PASS = self.userpassLineEdit.text
    
    #check on the server
    [connectToServer, connected, msg] =  self.logic.baseLogic.checkConnection()
    if connectToServer:
      if connected:
        self.loginButton.enabled = False
        self.logoutButton.enabled = True
        self.serverAdrsEdit.enabled = False
        self.serverPortEdit.enabled = False
        self.usernameLineEdit.enabled = False
        self.userpassLineEdit.enabled = False
        self.logic.baseLogic.LOGGEDIN = True
        self.delayDisplay("Login success!", msec = 1500)
        #slicer.util.messageBox('Login success!', windowTitle='test')
        #slicer.util.delayDisplay("Login success!", autoCloseMsec = 1500)
        #slicer.util.infoDisplay('Login success!', windowTitle='Info')
      
        self.logic.baseLogic.forceModuleReload("DataModule") #reload only data
        self.logic.baseLogic.switchToTab(slicer, "DataModule")
      else:
        if msg is not None and msg != '':
          self.delayDisplay(msg, 1500)
        else:
          self.delayDisplay("Invalid login information!", 1500)
    else:
      self.delayDisplay("Failed to communicate with the server!", msec = 1500)

  def onDisconnect(self):
    print("DEBUG: onDisconnect")
    if(slicer.util.confirmOkCancelDisplay(
        "Are you sure you want to logout? \n" + 
        "Any non-saved changes will be lost.", 
        windowTitle="Confirmation logout")):
      #if self.logic.baseLogic.APP_RUNNING:
      self.logic.baseLogic.exitProc(True)
      self.onExit()
      
      self.logoutButton.enabled = False
      self.loginButton.enabled = True
      self.serverAdrsEdit.enabled = True #self.developerMode
      self.serverPortEdit.enabled = True # self.developerMode
      self.usernameLineEdit.enabled = True
      self.userpassLineEdit.enabled = True
      self.logic.baseLogic.LOGGEDIN = False
      self.logic.baseLogic.resetSegModuleGlobalParams()
      self.logic.baseLogic.forceModuleReload(None) #reload all

  def onExit(self):
    slicer.mrmlScene.Clear()

# In case specific logic needs to be used here
class LoginModuleLogic(ScriptedLoadableModuleLogic):
  """This class should implement all the actual
  computation done by your module.  The interface
  should be such that other python code can import
  this class and make use of the functionality without
  requiring an instance of the Widget.
  Uses ScriptedLoadableModuleLogic base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """
  
  # constructor
  def __init__(self, baseLogic, instanceGUI):
    self.baseLogic = baseLogic
    self.instanceModuleGUI = instanceGUI


class LoginModuleTest(ScriptedLoadableModuleTest):
  """
  This is the test case for your scripted module.
  Uses ScriptedLoadableModuleTest base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setUp(self):
    """ Do whatever is needed to reset the state - typically a scene clear will be enough.
    """
    slicer.mrmlScene.Clear(0)

  def runTest(self):
    """Run as few or as many tests as needed here.
    """
    self.setUp()
    self.test_TestModuleTest1()

  def test_TestModuleTest1(self):
    """ Ideally you should have several levels of tests.  At the lowest level
    tests should exercise the functionality of the logic with different inputs
    (both valid and invalid).  At higher levels your tests should emulate the
    way the user would interact with your code and confirm that it still works
    the way you intended.
    One of the most important features of the tests is that it should alert other
    developers when their changes will have an impact on the behavior of your
    module.  For example, if a developer removes a feature that you depend on,
    your test should break so they know that the feature is needed.
    """

    pass
