# -*- coding: cp1252 -*-
import os, sys, time
import unittest
import requests
import vtk, qt, ctk, slicer, json 
from qt import QTableWidget, QTableWidgetItem
from PythonQt import Qt
# from PythonQt import QtCore
from slicer.ScriptedLoadableModule import *
import logging

#
# SILA3DModule - Model creation/modification tab
#

class ModelModule(ScriptedLoadableModule):
  """Uses ScriptedLoadableModule base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def __init__(self, parent):
    ScriptedLoadableModule.__init__(self, parent)
    self.parent.title = "SILA3D" # TODO make this more human readable by adding spaces
    self.parent.categories = ["SILA3D - Model management"]
    self.parent.dependencies = []
    self.parent.contributors = ["Barthelemy Serres (ILIAD3, Universit de Tours)", "Gaetan Galisot(LIFAT, Universit de Tours)"] # replace with "Firstname Lastname (Organization)"
    self.parent.helpText = """
This is the SILA3D python scripted module for slicer3D.
It performs a step by step segmentation on the input volume using an atlas, through a web service.
"""
    self.parent.helpText += self.getDefaultModuleDocumentationLink()
    self.parent.acknowledgementText = """
SILA3D : Segmentation Interactive using a Local Atlas
""" # replace with organization, grant and thanks.
    
    # M.D.
    self.baseLogic = None
#
# TemplateKeyWidget
#

class ModelModuleWidget(ScriptedLoadableModuleWidget):
  """Uses ScriptedLoadableModuleWidget base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setup(self):
    self.developerMode = False
  
    ScriptedLoadableModuleWidget.setup(self)
    self.logic = ModelModuleLogic(slicer.modules.ModelModuleInstance.baseLogic, self)
    
    # Instantiate and connect widgets ...
    # Model management
    
    self.ModelManagementCollapsibleButton = ctk.ctkCollapsibleButton()
    self.ModelManagementCollapsibleButton.text = "Anatomical model"
    self.layout.addWidget(self.ModelManagementCollapsibleButton)

    vLayout = qt.QVBoxLayout(self.ModelManagementCollapsibleButton)
    
    modelManagementLayout_1 = qt.QHBoxLayout()
    modelManagementLayout_2 = qt.QHBoxLayout()
    modelManagementLayout_3 = qt.QHBoxLayout()
    modelManagementLayout_4 = qt.QHBoxLayout()
    modelManagementLayout_5 = qt.QHBoxLayout()
    modelManagementLayout_6 = qt.QHBoxLayout()
    vLayout.addLayout(modelManagementLayout_1)
    vLayout.addLayout(modelManagementLayout_2)
    vLayout.addLayout(modelManagementLayout_3)
    vLayout.addLayout(modelManagementLayout_6)
    vLayout.addLayout(modelManagementLayout_4)
    vLayout.addLayout(modelManagementLayout_5)

    self.modelManagementHelpTextLbl = qt.QLabel()
    self.modelManagementHelpTextLbl.text = "Create a new anatomical model or update the loaded one.\nIf the model already exists, clicking create will generate a copy of it and apply the new updates."
    modelManagementLayout_1.addWidget(self.modelManagementHelpTextLbl)

    self.modelMsgLbl = qt.QLabel()
    self.modelMsgLbl.text = ""
    self.modelMsgLbl.setStyleSheet("""
        min-width: 80px;
        margin-top: 10px;
    """)
    modelManagementLayout_2.addWidget(self.modelMsgLbl)
    self.modelNameLbl = qt.QLabel()
    self.modelNameLbl.text = ""
    self.modelNameLbl.setStyleSheet("""
        min-width: 25px;
        margin-top: 10px;
        font-weight: bold;
        text-decoration: underline;
    """)
    modelManagementLayout_2.addWidget(self.modelNameLbl)
    modelManagementLayout_2.addStretch(1)
    
    self.newModelNameLbl = qt.QLabel()
    self.newModelNameLbl.text = "New model's name:"
    self.newModelNameLbl.setStyleSheet("""
        min-width: 80px;
        margin-top: 10px;
    """)
    self.newModelNameLbl.hide()
    modelManagementLayout_3.addWidget(self.newModelNameLbl)
    self.modelNewNameEdit = qt.QLineEdit("")
    self.modelNewNameEdit.text = ""
    self.modelNewNameEdit.setStyleSheet("""
        margin-top: 10px;
    """)
    self.modelNewNameEdit.setMinimumWidth(120)
    self.modelNewNameEdit.hide()
    modelManagementLayout_3.addWidget(self.modelNewNameEdit)
    modelManagementLayout_3.addStretch(1)
    
    # public or not
    self.cbxModelPublic = qt.QCheckBox()
    self.cbxModelPublic.setChecked(False)
    self.cbxModelPublic.setStyleSheet("""
        margin-left: 5px;
    """)
    self.cbxModelPublic.toolTip = "Available for Admin users"
    self.cbxModelPublicLbl = qt.QLabel()
    self.cbxModelPublicLbl.text = "Create a public MODEL?"
    self.cbxModelPublicLbl.setStyleSheet("""
        min-width: 25px;
    """)
    modelManagementLayout_6.addWidget(self.cbxModelPublicLbl)
    modelManagementLayout_6.addWidget(self.cbxModelPublic)
    modelManagementLayout_6.addStretch()
    
    #buttons
    self.createBtn = qt.QPushButton("Create")
    self.createBtn.connect('clicked(bool)', self.createModelAction)
    self.createBtn.setStyleSheet("""
        min-width: 80px;
        height: 19px;
        margin-top: 15px;
        margin-bottom: 10px;
    """)
    modelManagementLayout_4.addWidget(self.createBtn)
    self.updateBtn = qt.QPushButton("Update")
    self.updateBtn.connect('clicked(bool)', self.updateModelAction)
    self.updateBtn.setStyleSheet("""
        min-width: 80px;
        height: 19px;
        margin-top: 15px;
        margin-bottom: 10px;
    """)
    modelManagementLayout_4.addWidget(self.updateBtn)
    
    # push all controls to the right
    modelManagementLayout_5.setContentsMargins(0, 10, 0, 0)
    modelManagementLayout_5.addStretch()
    # processing msg
    self.msgProcessingLbl = qt.QLabel()
    self.msgProcessingLbl.text = "This task may take several minutes!"
    self.msgProcessingLbl.setStyleSheet("""
        color: #013387;
    """)
    self.msgProcessingLbl.hide()
    modelManagementLayout_5.addWidget(self.msgProcessingLbl)
    # processing progress
    self.percentageProcessingLbl = qt.QLabel()
    self.percentageProcessingLbl.text = "0%"
    self.percentageProcessingLbl.setStyleSheet("""
        color: #013387;
    """)
    self.percentageProcessingLbl.hide()
    modelManagementLayout_5.addWidget(self.percentageProcessingLbl)
    # Gif
    self.loadingGifLbl = qt.QLabel()
    self.loadingGifLbl.setFixedWidth(12)
    self.loadingGifLbl.setFixedHeight(12)
    moduleDir = os.path.dirname(slicer.util.modulePath(self.__module__))
    loadingiconPath = os.path.join(moduleDir, 'Resources/Imgs', 'ajax-loader.gif')
    self.loadingGifMovie = qt.QMovie(loadingiconPath);
    self.loadingGifMovie.setScaledSize(qt.QSize(12, 12))
    self.loadingGifLbl.setMovie(self.loadingGifMovie)
    self.loadingGifLbl.hide()
    self.loadingGifMovie.stop()
    modelManagementLayout_5.addWidget(self.loadingGifLbl)
    
    #######################################
    # Models regions Management
    #
    self.ModelDetailsCollapsibleButton = ctk.ctkCollapsibleButton()
    self.ModelDetailsCollapsibleButton.text = "Model details"
    self.layout.addWidget(self.ModelDetailsCollapsibleButton)

    vLayout_model = qt.QVBoxLayout(self.ModelDetailsCollapsibleButton)
    
    # Layout within the dummy collapsible button
    modelFormLayout_1 = qt.QHBoxLayout()
    vLayout_model.addLayout(modelFormLayout_1)
    modelFormLayout_2 = qt.QHBoxLayout()
    vLayout_model.addLayout(modelFormLayout_2)
    
    self.noRegionsLbl = qt.QLabel()
    self.noRegionsLbl.text = "There is no model loaded."
    self.noRegionsLbl.setStyleSheet("""
        min-width: 25px;
        margin-top: 10px;
    """)
    self.saveRegionsBtn = qt.QPushButton("Save")
    self.saveRegionsBtn.setStyleSheet("""
        height: 15px;
        margin-left: 4px;
        width: 70px;
    """)
    self.saveRegionsBtn.connect('clicked(bool)', self.saveRegionForModel)
    self.saveRegionsBtn.hide()
    self.modelManagHintLbl = qt.QLabel()
    self.modelManagHintLbl.text = "Double-click on a cell to edit its content.<br/><u>Warning</u>: do not change ID and Name at the same time, changes will not take effect.<br/><u>Warning</u>: Changing IDs may break existing segmentation sessions."
    self.modelManagHintLbl.setStyleSheet("""
        font-style: italic;
        color: #5665b7;
        font-size: 7px;
        margin-left: 4px;
    """)
    self.modelManagHintLbl.hide()
    modelFormLayout_1.addWidget(self.noRegionsLbl)
    modelFormLayout_1.addWidget(self.saveRegionsBtn)
    modelFormLayout_1.addWidget(self.modelManagHintLbl)
    modelFormLayout_1.addStretch(1)
    
    self.tableWidget = QTableWidget()
    self.tableWidget.hide()
    self.tableWidget.doubleClicked.connect(self.modelRegionsTablClick)
    modelFormLayout_2.addWidget(self.tableWidget)
    
    #######################################
    # Models history and logs
    #
    self.ModelLogsCollapsibleButton = ctk.ctkCollapsibleButton()
    self.ModelLogsCollapsibleButton.text = "History and logs"
    self.layout.addWidget(self.ModelLogsCollapsibleButton)

    vLayout_1 = qt.QVBoxLayout(self.ModelLogsCollapsibleButton)
    
    modelLogsLayout_1 = qt.QHBoxLayout()
    vLayout_1.addLayout(modelLogsLayout_1)
    modelLogsLayout_2 = qt.QHBoxLayout()
    vLayout_1.addLayout(modelLogsLayout_2)
    
    self.modelLogsHelpTextLbl = qt.QLabel()
    self.modelLogsHelpTextLbl.text = "History of updates applied to the model."
    modelLogsLayout_1.addWidget(self.modelLogsHelpTextLbl)
    
    self.modelLogsLbl = qt.QLabel()
    self.modelLogsLbl.text = "There are no recorded logs to show."
    self.modelLogsLbl.setStyleSheet("""
        min-width: 25px;
        margin-top: 10px;
    """)
    modelLogsLayout_2.addWidget(self.modelLogsLbl)
    
    # Add vertical spacer
    self.layout.addStretch(1)
    
    #init
    self.initUIForModelModule()
    
  
  def cleanup(self):
    pass

  def delayDisplay(self,message,msec=100):
    """This utility method displays a small dialog and waits.
    This does two things: 1) it lets the event loop catch up
    to the state of the test so that rendering and widget updates
    have all taken place before the test continues and 2) it
    shows the user/developer/tester the state of the test
    so that we'll know when it breaks.
    """
    print(message)
    self.info = qt.QDialog()
    self.info.setWindowTitle('Info')
    if self.logic.baseLogic.WICON is not None:
      self.info.setWindowIcon(self.logic.baseLogic.WICON)
    self.info.setWindowFlags(qt.Qt.WindowTitleHint)
    self.info.resize(150,30)
    self.infoLayout = qt.QVBoxLayout()
    self.info.setLayout(self.infoLayout)
    self.label = qt.QLabel(message,self.info)
    self.infoLayout.addWidget(self.label, qt.Qt.AlignCenter)
    qt.QTimer.singleShot(msec, self.info.close)
    self.info.exec_()

  def initUIForModelModule(self):
    self.cbxModelPublic.setChecked(False)
    # check if logged in and the app is running
    if (self.logic.baseLogic.LOGGEDIN and 
        self.logic.baseLogic.APP_RUNNING):
      # change msg to display the model's name
      if self.logic.baseLogic.CURRENT_MODEL == "NEW":
        self.modelMsgLbl.text = "Enter new model's name: "
        self.modelMsgLbl.hide()
        self.modelNameLbl.text = ""
        self.modelNameLbl.hide()
        self.modelNewNameEdit.show()
        self.newModelNameLbl.show()
        self.updateBtn.enabled = False
      else:
        self.modelMsgLbl.text = 'Active model: '
        self.modelMsgLbl.show()
        self.modelNameLbl.text = self.logic.baseLogic.CURRENT_MODEL
        self.modelNameLbl.show()
        self.modelNewNameEdit.show()
        self.newModelNameLbl.show()
        if self.logic.baseLogic.CURRENT_MODEL_PUBLIC and self.logic.baseLogic.USER_ROLE != 'admin':
          self.updateBtn.enabled = False
        else:
          self.updateBtn.enabled = True
      if self.logic.baseLogic.USER_ROLE == "admin":
        self.cbxModelPublic.enabled = True
      else:
        self.cbxModelPublic.enabled = False
      self.toggleUIControls(True)
    else:
      self.modelMsgLbl.text = 'Active model: '
      self.modelMsgLbl.show()
      self.modelNameLbl.text = 'No model'
      self.modelNameLbl.show()
      self.modelNewNameEdit.hide()
      self.newModelNameLbl.hide()
      self.cbxModelPublic.enabled = False
      self.toggleUIControls(False)
    
    # Populate table with regions
    self.populateTableWidgetModelRegions()
    
    # logs
    self.hanldeModelLogs()

  def toggleUIControls(self, on):
    if on:
      self.ModelManagementCollapsibleButton.enabled = True
      self.ModelLogsCollapsibleButton.enabled = True
      self.ModelDetailsCollapsibleButton.enabled = True
    else:
      self.ModelManagementCollapsibleButton.enabled = False
      self.ModelLogsCollapsibleButton.enabled = False
      self.ModelDetailsCollapsibleButton.enabled = False
  
  def populateTableWidgetModelRegions(self):
    print("Entering populateTableWidgetModelRegions in ModelModule")
    if (self.logic.baseLogic.LOGGEDIN and 
        self.logic.baseLogic.APP_RUNNING):
      numR = len(self.logic.baseLogic.REGION_DICTIONARY)
      if (numR > 0):
        print("Data has been found for table")
        self.noRegionsLbl.hide()
        self.tableWidget.show()
        self.saveRegionsBtn.show()
        self.modelManagHintLbl.show()
        # set column count
        self.tableWidget.setColumnCount(2)
        self.tableWidget.setHorizontalHeaderLabels(['ID', 'Name'])
        # set row count
        self.tableWidget.setRowCount(numR)
        row = 0
        for i, v in self.logic.baseLogic.REGION_DICTIONARY.items():
          item = QTableWidgetItem()
          item.setData(Qt.Qt.DisplayRole, i) # 0 means DisplayRole
          item.setText(str(i))
          self.tableWidget.setItem(row, 0, item)
          self.tableWidget.setItem(row, 1, QTableWidgetItem(v))
          row = row + 1
        self.tableWidget.horizontalHeader().setStretchLastSection(True);
        self.tableWidget.setSortingEnabled(True)
      else:
        self.tableWidget.setRowCount(0)
        self.tableWidget.clear()
        self.tableWidget.hide()
        self.saveRegionsBtn.hide()
        self.noRegionsLbl.show()
        self.modelManagHintLbl.hide()
        print("No data")
    else:
      self.tableWidget.setRowCount(0)
      self.tableWidget.clear()
      self.tableWidget.hide()
      self.saveRegionsBtn.hide()
      self.modelManagHintLbl.hide()
      self.noRegionsLbl.show()
      print("No model is loaded")
    print("Exiting populateTableWidgetModelRegions in ModelModule")

  def hanldeModelLogs(self):
    if (self.logic.baseLogic.LOGGEDIN and
        self.logic.baseLogic.APP_RUNNING):
      # actual model is loaded
      if self.logic.baseLogic.CURRENT_MODEL != "NEW":
        # get the logs
        [found, msg, logs] = self.logic.getModelLogs(self.logic.baseLogic.CURRENT_MODEL_FLD)
        if found:
          # store the logs into logic
          self.logic.MODEL_LOGS = logs
          # construct logs to show
          text = ""
          text += "Model: <font color='blue'>" + logs["name"] + "</font><br/>"
          text += "Model last modified by: " + str(logs["lastModifiedBy"]) + "<br/>"
          text += "Model last modified time: " + logs["lastModifiedTime"] + "<br/>"
          text += "Regions: <br/>"
          for region in logs["regions"]:
            text += "  History for region <b><font color='blue'>" + str(region["ID"]) + "</font></b><br/>"
            i = 0
            for img in region["imageName"]:
              text += "    " + "<font color='blue'>" + img + "</font>" + "  " + str(region["userModif"][i]) + "  " + region["dateTimeModif"][i] + "<br/>"
              i = i + 1
          self.modelLogsLbl.text = text
        else:
          if msg != '':
            self.delayDisplay(msg,4000)
          # store the logs into logic
          self.logic.MODEL_LOGS = None
          self.modelLogsLbl.text = "There are no recorded logs to show.<br/>[]"

  def modelRegionsTablClick(self):
    # msg = ""
    # for currentQTableWidgetItem in self.tableWidget.selectedItems():
      # msg = (str(currentQTableWidgetItem.row()) +
        # str(currentQTableWidgetItem.column()) +
        # currentQTableWidgetItem.text())
    # self.delayDisplay(msg, 1500)
    self.logic.MODELS_REGIONS_MODIFIED = True
    pass
  
  def onExit(self):
    slicer.mrmlScene.Clear()
    
  def createModelAction(self):
    if self.logic.baseLogic.LOADED_SESSION_DIRNAME == "":
      self.delayDisplay("There is no loaded session! Cannot create/update the model.", 1500)
      return
    # two cases to distinguish between
    name = self.modelNewNameEdit.text
    if (name == ""):
      self.delayDisplay("Please enter a name!", 1500)
      return
    
    # verify new name is unique locally
    # for mdl in self.logic.baseLogic.MODELS:
      # if (mdl.lower() == name.lower()):
        # self.delayDisplay("A model with the same name already exists.\nPlease enter a new name!", 1500)
        # return
    # verify new name is unique on server
    if not self.logic.verfiyNewModelName(name.lower()):
      self.delayDisplay("A model with the same name already exists.\nPlease enter a new name!", 1500)
      return
    
    makePublic = False
    if self.cbxModelPublic.isChecked() and self.logic.baseLogic.USER_ROLE == 'admin':
      makePublic = True
    
    # case 1: new model ==> learn
    if (self.logic.baseLogic.CURRENT_MODEL == "NEW"):
      # call API to start learning
      self.startLearning(name, "NONE", [], makePublic)
    else: #case 2 ==> ask for confirmation 
      self.mypopup = customUpdatePopup(
        self.logic.baseLogic.CURRENT_MODEL,
        self.logic.baseLogic.CURRENT_T1,
        self.logic.baseLogic.CURRENT_T1_FILE,
        self.logic,
        self,
        name, self.cbxModelPublic.isChecked())
      self.mypopup.show()
      # if (slicer.util.confirmOkCancelDisplay(
            # "This action will create a new model only with the current segmented regions." +
            # "\nDo you wish to proceed?", 
            # windowTitle="Confirm copying model")):
        # pass
      # else:
        # return
  
  def updateModelAction(self):
    if self.logic.baseLogic.LOADED_SESSION_DIRNAME == "":
      self.delayDisplay("There is no loaded session! Cannot create/update the model.", 1500)
      return
    if (self.logic.baseLogic.CURRENT_MODEL == "NEW"):
      self.delayDisplay("Can't update a non existing model!", 1500)
    else:
      if self.logic.baseLogic.USER_ROLE == 'admin' and self.logic.baseLogic.CURRENT_MODEL_PUBLIC:
        if (slicer.util.confirmOkCancelDisplay(
            "You are about to update a public model. Do you with to proceed?", 
            windowTitle="Confirm update public model")):
          self.mypopup = customUpdatePopup(
            self.logic.baseLogic.CURRENT_MODEL,
            self.logic.baseLogic.CURRENT_T1,
            self.logic.baseLogic.CURRENT_T1_FILE,
            self.logic,
            self,
            "", True)
          self.mypopup.show()
        return
      if self.logic.baseLogic.USER_ROLE != 'admin' and self.logic.baseLogic.CURRENT_MODEL_PUBLIC:
        self.delayDisplay("You are not authorized to perform this update", 1500)
        return
      
      self.mypopup = customUpdatePopup(
        self.logic.baseLogic.CURRENT_MODEL,
        self.logic.baseLogic.CURRENT_T1,
        self.logic.baseLogic.CURRENT_T1_FILE,
        self.logic,
        self,
        "", False)
      self.mypopup.show()
          
      # this action is now done by the btnSubmit in popup
      # self.startLearning(self.logic.baseLogic.CURRENT_MODEL)
  
  def startLearning(self, name, action, customR, isPublic, copyFrom="", copyPublic=False):
    if self.logic.baseLogic.LOADED_SESSION_DIRNAME == "":
      self.delayDisplay("There is no loaded session! Cannot create/update the model.", 1500)
      return
    print("start learning")
    if self.logic.startLearning(name, action, customR, isPublic, copyFrom, copyPublic):
     #start the timer and keep calling the check api to ge the status
      self.logic.baseLogic.LEARN_TIMER.timeout.connect(self.checkLearningStatus)
      self.logic.baseLogic.LEARN_TIMER.start(10000)
      print("learning timer has started")
      #TODO lock the UI
      self.toggleUIControls(False)
      # show icon loading
      self.showHideLoading(True, 0)
      
      self.delayDisplay("Model generation has started!", 1500)
    else:
      self.delayDisplay("Model generation could not start. Please try again!", 1500)
  
  def checkLearningStatus(self):
    # call the api
    success, done, msg = self.logic.checkLearningStatus()
    if done:
      if self.logic.baseLogic.LEARN_TIMER != None and self.logic.baseLogic.LEARN_TIMER.isActive():
        self.logic.baseLogic.LEARN_TIMER.stop()
        self.logic.baseLogic.LEARN_TIMER = qt.QTimer()
        print("Learning timer is stopped")
        self.logic.baseLogic.LEARN_CNT_TIMER_RAN = 100
        self.showHideLoading(True, 100)
        self.endLearning(success, msg)
    else: # not done so increase the counter per call
      if self.logic.baseLogic.LEARN_CNT_TIMER_RAN == 90: # stop increasing and wait
        pass
      else:
        self.logic.baseLogic.LEARN_CNT_TIMER_RAN = self.logic.baseLogic.LEARN_CNT_TIMER_RAN + 10
        self.showHideLoading(True, self.logic.baseLogic.LEARN_CNT_TIMER_RAN)
  
  def endLearning(self, success, msg):
    print("Learning is done.")
    
    self.showHideLoading(False, 0)
    self.logic.baseLogic.LEARN_CNT_TIMER_RAN = 0
    
    # unlock the UI
    self.toggleUIControls(True)
    
    if (not success):
      self.delayDisplay("Failed to create the model.\ndetails:" + msg, 1500)
      self.logic.baseLogic.resetSegModuleGlobalParams()
      self.logic.baseLogic.exitProc(False)
      self.logic.baseLogic.forceModuleReload("DataModule")
      self.logic.baseLogic.forceModuleReload("SegModule")
      self.logic.baseLogic.forceModuleReload("ModelModule")
      self.logic.baseLogic.switchToTab(slicer, "DataModule")
    else:
      if self.logic.baseLogic.CURRENT_MODEL == "NEW":
        self.logic.finalizeNewModelAction(self.modelNewNameEdit.text)
      if (slicer.util.confirmOkCancelDisplay(
            "Model generation has been done!" +
            "\nIf a new model has been created, it has to be loaded to be used in future segmentations." +
            "\nWould you like to load the new model?", 
            windowTitle="Confirm reload model")):
        self.logic.baseLogic.resetSegModuleGlobalParams()
        self.logic.baseLogic.exitProc(False)
        self.logic.baseLogic.forceModuleReload("DataModule")
        self.logic.baseLogic.forceModuleReload("SegModule")
        self.logic.baseLogic.forceModuleReload("ModelModule")
        self.logic.baseLogic.switchToTab(slicer, "DataModule")
  
  def showHideLoading(self, show, perc):
    if show:
      self.loadingGifMovie.start()
      self.loadingGifLbl.show()
      self.percentageProcessingLbl.text = str(perc) + "%"
      self.percentageProcessingLbl.show()
      self.msgProcessingLbl.show()
    else:
      self.loadingGifMovie.stop()
      self.loadingGifLbl.hide()
      self.percentageProcessingLbl.text = ""
      self.percentageProcessingLbl.hide()
      self.msgProcessingLbl.hide()

  def saveRegionForModel(self):
    if self.logic.MODELS_REGIONS_MODIFIED:
      # collect all data from the table
      newRegions = {}
      rows = self.tableWidget.rowCount
      for r in xrange(0, rows):
        idS = self.tableWidget.item(r, 0).text()
        name = self.tableWidget.item(r, 1).text()
        id = -1
        canCast = False
        try:
          id = int(idS)
          canCast = True
        except ValueError:
          canCast = False
        
        if not canCast or id < 0 or id in newRegions or name == "" or name in newRegions.values():
          self.delayDisplay("Wrong data entered in table! Save cannot be done.\n(check row " + str(r + 1) + ")", 1500)
          return
        
        newRegions.update({id: name})
        
      # post them to the server
      model = ""
      if self.logic.baseLogic.CURRENT_MODEL == "NEW":
        model = self.modelNewNameEdit.text
      else:
        model = self.logic.baseLogic.CURRENT_MODEL
      if self.logic.saveRegionsForModel(newRegions, model):
        self.logic.MODELS_REGIONS_MODIFIED = False
        if (slicer.util.confirmOkCancelDisplay(
            "Saving the new regions data has been done!" +
            "\nYou need to reload the model to see the new information." +
            "\nWould you like to reload the model?", 
            windowTitle="Confirm reload model")):
          #tmp data
          cmodel = model
          cmodel_pub = self.logic.baseLogic.CURRENT_MODEL_PUBLIC
          cmodel_fld = self.logic.baseLogic.CURRENT_MODEL_FLD
          cT1 = self.logic.baseLogic.CURRENT_T1
          cT1_pub = self.logic.baseLogic.CURRENT_T1_PUBLIC
          cT1_file = self.logic.baseLogic.CURRENT_T1_FILE
          cmodels = self.logic.baseLogic.MODELS
          segReg = self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION
          self.logic.baseLogic.resetSegModuleGlobalParams()
          self.logic.baseLogic.CURRENT_MODEL = cmodel
          self.logic.baseLogic.CURRENT_MODEL_PUBLIC = cmodel_pub
          self.logic.baseLogic.CURRENT_MODEL_FLD = cmodel_fld
          self.logic.baseLogic.CURRENT_T1 = cT1
          self.logic.baseLogic.CURRENT_T1_PUBLIC = cT1_pub
          self.logic.baseLogic.CURRENT_T1_FILE = cT1_file
          self.logic.baseLogic.MODELS = cmodels
          self.logic.baseLogic.APP_RUNNING = True
          self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION = segReg
          self.logic.baseLogic.exitProc(False)
          self.logic.baseLogic.forceModuleReload("SegModule")
          self.logic.baseLogic.forceModuleReload("ModelModule")
          self.logic.baseLogic.switchToTab(slicer, "SegModule")
      else:
        self.delayDisplay("An error has occurred while saving. Please try again!", 1500)
    else:
      self.delayDisplay("No changes need saving!", 1500)


#
# PopupWindow class
#
class customUpdatePopup(qt.QWidget):
  def __init__(self, mdl, img, imgFname, lgic, instMdl, newname, isPublic):
    qt.QWidget.__init__(self)
    self.setWindowTitle("Update model")
    self.model = mdl
    self.image = img
    self.imageFileName = imgFname
    self.logic = lgic
    self.instanceModule = instMdl
    self.newModelName = newname
    self.isPublic = isPublic
    
    self.drawUI()

  def paintEvent(self, e):
    pass

  def drawUI(self):
    self.vbox = qt.QVBoxLayout()
    self.setLayout(self.vbox)
    
    self.hbox_1 = qt.QHBoxLayout()
    self.vbox.addLayout(self.hbox_1)
    self.generalLbl = qt.QLabel()
    self.generalLbl.text = "Anatomical model <b><u>" + self.model + "</u></b> will be updated.<br/><br/>"
    self.generalLbl.setStyleSheet("""
        min-width: 25px;
    """)
    self.hbox_1.addWidget(self.generalLbl)
    
    self.hbox_2 = qt.QHBoxLayout()    
    self.vbox.addLayout(self.hbox_2)
    self.newRLbl = qt.QLabel()
    self.newRLbl.text = "New regions to learn and add to the model are: "
    self.newRLbl.setStyleSheet("""
        min-width: 25px;
        margin-left: 8px;
    """)
    self.hbox_2.addWidget(self.newRLbl)
    
    self.hbox_3 = qt.QHBoxLayout()    
    self.vbox.addLayout(self.hbox_3)
    self.newRVLbl = qt.QLabel()
    self.newRVLbl.text = ""
    self.newRVLbl.setStyleSheet("""
        min-width: 25px;
        margin-left: 12px;
    """)
    self.hbox_3.addWidget(self.newRVLbl)
    
    self.hbox_4 = qt.QHBoxLayout()    
    self.vbox.addLayout(self.hbox_4)
    self.exclRLbl = qt.QLabel()
    self.exclRLbl.text = "Regions already learnt based on image <u>" + self.image + "</u> to be excluded are: "
    self.exclRLbl.setStyleSheet("""
        min-width: 25px;
        margin-left: 8px;
    """)
    self.hbox_4.addWidget(self.exclRLbl)
    
    self.hbox_5 = qt.QHBoxLayout()    
    self.vbox.addLayout(self.hbox_5)
    self.exclRVLbl = qt.QLabel()
    self.exclRVLbl.text = ""
    self.exclRVLbl.setStyleSheet("""
        min-width: 25px;
        margin-left: 12px;
    """)
    self.hbox_5.addWidget(self.exclRVLbl)
    
    self.hbox_6 = qt.QHBoxLayout()    
    self.vbox.addLayout(self.hbox_6)
    self.breakLinesLbl = qt.QLabel()
    self.breakLinesLbl.text = "<br/><br/><br/>"
    self.hbox_6.addWidget(self.breakLinesLbl)
    
    self.hbox_7 = qt.QHBoxLayout()
    self.vbox.addLayout(self.hbox_7)
    self.customizeLbl = qt.QLabel()
    self.customizeLbl.text = "Customize regions to be learn:"
    self.customizeLbl.setStyleSheet("""
        min-width: 25px;
    """)
    self.hbox_7.addWidget(self.customizeLbl)
    
    self.hbox_8 = qt.QHBoxLayout()
    self.vbox.addLayout(self.hbox_8)
    self.noCustomzieRB = qt.QRadioButton("No customization")
    self.noCustomzieRB.setChecked(True)
    self.includeAllRB = qt.QRadioButton("Include all")
    self.includeRB = qt.QRadioButton("Include some")
    self.excludeRB = qt.QRadioButton("Exclude some")
    self.noCustomzieRB.toggled.connect(lambda: self.rbClicked(self.noCustomzieRB))
    self.includeAllRB.toggled.connect(lambda: self.rbClicked(self.includeAllRB))
    self.includeRB.toggled.connect(lambda: self.rbClicked(self.includeRB))
    self.excludeRB.toggled.connect(lambda: self.rbClicked(self.excludeRB))
    self.hbox_8.addWidget(self.noCustomzieRB)
    self.hbox_8.addWidget(self.includeAllRB)
    self.hbox_8.addWidget(self.includeRB)
    self.hbox_8.addWidget(self.excludeRB)
    
    self.hbox_9 = qt.QHBoxLayout()
    self.vbox.addLayout(self.hbox_9)
    self.customLbl = qt.QLabel()
    self.customLbl.text = "Enter regions IDs separated by comma: "
    self.customLbl.setStyleSheet("""
        min-width: 25px;
    """)
    self.customTbx = qt.QLineEdit("")
    self.customTbx.text = ""
    self.customTbx.setMinimumWidth(120)
    self.customTbx.enabled = False
    self.hbox_9.addWidget(self.customLbl)
    self.hbox_9.addWidget(self.customTbx)
    self.hbox_9.addStretch()
    
    self.hbox_10 = qt.QHBoxLayout()
    self.vbox.addLayout(self.hbox_10)
    self.submitBtn = qt.QPushButton("Update")
    self.submitBtn.setStyleSheet("""
        min-width: 80px;
        height: 17px;
        margin-top: 15px;
        background-color: #A9C5D7;
    """)
    self.submitBtn.connect('clicked(bool)', self.submitBtnClicked)
    self.cancelBtn = qt.QPushButton("Cancel")
    self.cancelBtn.setStyleSheet("""
        min-width: 80px;
        height: 17px;
        margin-top: 15px;
    """)
    self.cancelBtn.connect('clicked(bool)', self.cancelBtnClicked)
    self.hbox_10.addStretch(1)
    self.hbox_10.addWidget(self.submitBtn)
    self.hbox_10.addWidget(self.cancelBtn)
    
    self.vbox.addStretch()
    
    self.setGeometry(qt.QRect(200, 100, 400, 200))
    
    # init data
    [newR, tR] = self.getNewRegionsToLearn()
    self.newR = newR
    self.newRVLbl.text = tR
    [exclR, t] = self.getExcludedRegions()
    self.exclR = exclR
    self.exclRVLbl.text = t
  
  def rbClicked(self, rb):
    txt = rb.text
    chckd = rb.isChecked()
    if chckd and (txt == "No customization" or txt == "Include all"):
      self.customTbx.enabled = False
    if chckd and (txt == "Include some" or txt == "Exclude some"):
      self.customTbx.enabled = True
      self.customTbx.setFocus()
  
  def submitBtnClicked(self):
    # get the checked rb
    customAction = ""
    customR = []
    valid = True
    if self.noCustomzieRB.isChecked():
      customAction = "NONE"
    if self.includeAllRB.isChecked():
      customAction = "ALL"
    if self.includeRB.isChecked():
      customAction = "INC"
    if self.excludeRB.isChecked():
      customAction = "EXCL"
    
    if customAction == "INC" or customAction == "EXCL":
      # parse regions
      if self.customTbx.text == "":
        valid = False
        self.instanceModule.delayDisplay("Enter regions separated by comma.", 1500)
        self.customTbx.setFocus()
      else:
        rgs = self.customTbx.text.split(",")
        if (all(r.isdigit() for r in rgs)):
          customR = [int(r) for r in rgs]
          customR.sort()
        else:
          valid = False
          self.instanceModule.delayDisplay("Invalid syntax!\nPlease enter region IDs separated by a comma", 1500)
        
    print(valid)
    print(customAction)
    print(customR)
    
    # more validation
    if customAction == "INC" and not all(e in self.exclR  for e in customR):
      # check if anything in customR is not in exclR
      self.instanceModule.delayDisplay("Some included regions are wrong! (they are not in the excluded list)", 1500)
      return
      
    if customAction == "EXCL" and not all(e in self.newR  for e in customR):
      # check if anything in customR is not in exclR
      self.instanceModule.delayDisplay("Some excluded regions are wrong! (they are not in the new list)", 1500)
      return
    
    if customAction == "NONE" and self.newR == self.exclR:
      # if all excluded, there no point is submitting request
      self.instanceModule.delayDisplay("No new region to learn and no existing regions to re-learn!", 1500)
      return
    
    if customAction == "EXCL" and customR == self.newR:
      # the user cannot exclude all regions in new
      self.instanceModule.delayDisplay("No new region to learn and no existing regions to re-learn!", 1500)
      return
    
    # self.instanceModule.delayDisplay("Learn", 500)
    if valid:
      if self.newModelName != "": # copying the model
        self.instanceModule.startLearning(self.newModelName, customAction, customR, self.isPublic, self.logic.baseLogic.CURRENT_MODEL_FLD, self.logic.baseLogic.CURRENT_MODEL_PUBLIC)
      else: # updating existing model
        self.instanceModule.startLearning(self.logic.baseLogic.CURRENT_MODEL_FLD, customAction, customR, self.isPublic)
      self.close()
    
  def cancelBtnClicked(self):
    self.close()

  def getNewRegionsToLearn(self):
    newR = []
    text = ''
    # current model regions - excluded ones
    if (self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION != None or
        len(self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION) > 0):
      currR = self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION
      [exclR, t] = self.getExcludedRegions()
      for r in currR:
        if r not in exclR:
          newR.append(r)
      if len(newR) == 0:
        text = "No new regions to learn."
      else:
        newR.sort()
        for i in range(len(newR)):
          text += str(newR[i])
          if i < len(newR) - 1:
            text += ',' 
          if i != 0 and i % 20 == 0:
            text += '\n'
        #text = ",".join(map(str, newR))
    else:
      text = "No regions in the current segmentation."
    
    return [newR, text]
  
  def getExcludedRegions(self): #excluded regions should be based on regions in session
    currR = []
    if (self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION != None or
        len(self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION) > 0):
      currR = self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION
    exclR = []
    text = ""
    if self.logic.MODEL_LOGS != None:
      logs = self.logic.MODEL_LOGS
      # extract regions where image appears in its logs
      for region in logs["regions"]:
        if region["ID"] not in currR: # skip it 
          continue
        if self.imageFileName in region["imageName"]:
          exclR.append(region["ID"])
      if len(exclR) == 0:
        text = "No regions will be excluded."
      else:
        exclR.sort()
        for i in range(len(exclR)):
          text += str(exclR[i])
          if i < len(exclR) - 1:
            text += ',' 
          if i != 0 and i % 20 == 0:
            text += '\n'
        #text = ",".join(map(str, exclR))
    else:
      text = "No regions will be excluded."
    
    return [exclR, text]


#
# TestModuleLogic
#

class ModelModuleLogic(ScriptedLoadableModuleLogic):
  """This class should implement all the actual
  computation done by your module.  The interface
  should be such that other python code can import
  this class and make use of the functionality without
  requiring an instance of the Widget.
  Uses ScriptedLoadableModuleLogic base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """
  
  # constructor
  def __init__(self, baseLogic, instanceGUI):
    self.baseLogic = baseLogic
    self.instanceModuleGUI = instanceGUI
    self.baseLogic.LEARN_TIMER = qt.QTimer()
    self.MODELS_REGIONS_MODIFIED = False
    self.MODEL_LOGS = None
  
  def startLearning(self, name, action, customR, isPublic, copyFrom="", copyPublic=False):
    # call API to learn/create a model
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/learning/start/" + name + "/" + self.baseLogic.LOADED_SESSION_DIRNAME +
            "?makepublic=" + str(isPublic) + "&copyfrom=" + copyFrom + "&copypublic=" + str(copyPublic) + 
            "&action=" + action + "&customR=")
    if (len(customR) > 0):
      req = req + ','.join([str(r) for r in customR])
    print(req)
    ret = requests.get(req)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        return True
      else:
        return False
  
  def checkLearningStatus(self):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/process/check/learning")
    
    ret = requests.get(req)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        if resp["done"]:
          return [True, True, resp["message"]]
        else:
          return [True, False, resp["message"]]
      else:
        if resp["done"]:
          return [False, True, resp["message"]]
        else:
          return [False, False, resp["message"]]
          
  def finalizeNewModelAction(self, newModel):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/finalizenewmodel/" + newModel)
    ret = requests.get(req)
    print("Call to finalizeNewModel, status_code = " + str(ret.status_code))
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        return True
    
    return False
    
  def saveRegionsForModel(self, newRegions, model):
    arrReg = []
    for k, v in newRegions.items():
      t = {}
      t.update({"id": k})
      t.update({"name": v})
      arrReg.append(t)
    print(json.dumps(newRegions))
    print(json.dumps(arrReg))
    
    if len(arrReg) == 0:
      return False
    
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/" + model + "/regions")
    headers = {'Content-Type': 'application/json', 'Accept':'application/json'}
    ret = requests.post(req, data = json.dumps(arrReg), headers = headers)
    print("Call to update regions API, status_code = " + str(ret.status_code))
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        return True
    
    return False
    
  def getModelLogs(self, model):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/" + model + "/logs")
    ret = requests.get(req)
    print("Call to getModelLogs API, status_code = " + str(ret.status_code))
    print(ret.json())
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        return [True, resp["message"], resp["logs"]]
      else:
        return [False, '', None]
    else:
      [msg, retry] = self.baseLogic.hanldeFatalError(ret)
      if msg == "custom":
        msg = "Failed to get model logs because the server is busy executing the last task!\nLogs will be requested automatically when the server is done."
      #self.delayDisplay(msg, 4000)
      print(retry)
      if retry:
        print(self.baseLogic.TABS_FORCE_RELOAD)
        self.baseLogic.forceModuleReload("ModelModule")
        print(self.baseLogic.TABS_FORCE_RELOAD)
      return [False, msg, None]
    
  def verfiyNewModelName(self, newname):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
          "/neurobrainweb/" + self.baseLogic.USER_ID + 
          "/model/verifyname/" + newname)
    ret = requests.get(req)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        return True
    return False

class ModelModuleTest(ScriptedLoadableModuleTest):
  """
  This is the test case for your scripted module.
  Uses ScriptedLoadableModuleTest base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setUp(self):
    """ Do whatever is needed to reset the state - typically a scene clear will be enough.
    """
    slicer.mrmlScene.Clear(0)

  def runTest(self):
    """Run as few or as many tests as needed here.
    """
    self.setUp()
    self.test_TestModuleTest1()

  def test_TestModuleTest1(self):
    """ Ideally you should have several levels of tests.  At the lowest level
    tests should exercise the functionality of the logic with different inputs
    (both valid and invalid).  At higher levels your tests should emulate the
    way the user would interact with your code and confirm that it still works
    the way you intended.
    One of the most important features of the tests is that it should alert other
    developers when their changes will have an impact on the behavior of your
    module.  For example, if a developer removes a feature that you depend on,
    your test should break so they know that the feature is needed.
    """

    pass
