# -*- coding: cp1252 -*-
import os, sys, time
import unittest
import requests
import vtk, qt, ctk, slicer, json 
from slicer.ScriptedLoadableModule import *
import logging
from requests_toolbelt.multipart import encoder
import nrrd
import nibabel as nib
import numpy as np
import collections
if sys.version_info[0] >= 3:
    from urllib.request import urlretrieve
else:
    from urllib import urlretrieve

#
# SILA3DModule - Segmentation tab
#

class BBox():

  def __init__(self, origX, origY, origZ, sizeX, sizeY, sizeZ):
    self.origX=origX
    self.origY=origY
    self.origZ=origZ
    self.sizeX=sizeX
    self.sizeY=sizeY
    self.sizeZ=sizeZ

    self.origX_mm=origX*0.3 - 86
    self.origY_mm=origY*0.3 - 132
    self.origZ_mm=origZ*0.3 - 32

    self.sizeX_mm=sizeX*0.3
    self.sizeY_mm=sizeY*0.3
    self.sizeZ_mm=sizeZ*0.3

    self.node = None

  def position(self):
    return (self.x,self.y,self.z)

  def setPosition(self, x, y, z):
    self.origX=x
    self.origY=y
    self.origZ=z

  def setPosition_mm(self, x, y, z):
    self.origX_mm=x
    self.origY_mm=y
    self.origZ_mm=z 

  def size(self):
    return (self.sizeX,self.sizeY,self.sizeZ)

  def setSize(self, sX, sY, sZ):
    self.sizeX=sX
    self.sizeY=sY
    self.sizeZ=sZ

  def setSize_mm(self, sX, sY, sZ):
    self.sizeX_mm=sX
    self.sizeY_mm=sY
    self.sizeZ_mm=sZ

  def resetToDefault(self):
    self.origX=0
    self.origZ=0
    self.origY=0
    self.sizeX=10
    self.sizeY=10
    self.sizeZ=10

  def compute(self, spacing, orig, IJKToRAS):
    print("Entering compute in BBox")
    if self.node:
      print("Node is found in compute in BBox")
      self.IJKToRAS(spacing, orig, IJKToRAS)
      # nodeVolume = slicer.util.getFirstNodeByClassByName("vtkMRMLScalarVolumeNode", "B1_3_template")
     
      # matrix = vtk.vtkMatrix4x4()
      # nodeVolume.GetIJKToRASMatrix(matrix)
      # print "ijkToRAS:"+matrix
      # spacing = nodeVolume.GetSpacing()
      # print "spacing:"+ str(spacing[0])+str(spacing[1])+str(spacing[2])
     
      print("orig in mm:")
      print([self.origX_mm, self.origY_mm, self.origZ_mm])

      print("size in mm:")
      print([self.sizeX_mm, self.sizeY_mm, self.sizeZ_mm])

      self.node.SetXYZ([self.origX_mm, self.origY_mm, self.origZ_mm]);
      self.node.SetRadiusXYZ(self.sizeX_mm, self.sizeY_mm, self.sizeZ_mm)
    print("Exiting compute in BBox")

  def IJKToRAS(self, spacing, orig, IJKToRAS):

    print('orig : ', orig[0]  , orig[1]  ,  orig[2])
    print('space : ', spacing[0]  , spacing[1]  ,  spacing[2])

    Ori = [self.origX , self.origY, self.origZ, 1]
    Fin = [self.origX + self.sizeX, self.origY + self.sizeY, self.origZ + self.sizeZ, 1]
    ori_RAS = [0,0,0,1]
    Fin_RAS = [0,0,0,1]

    ori_RAS = IJKToRAS.MultiplyPoint(Ori)   
    Fin_RAS = IJKToRAS.MultiplyPoint(Fin)   

    self.origX_mm = (Fin_RAS[0] - ori_RAS[0])/2.0 + ori_RAS[0]
    self.origY_mm = (Fin_RAS[1] - ori_RAS[1])/2.0 + ori_RAS[1]
    self.origZ_mm = (Fin_RAS[2] - ori_RAS[2])/2.0 + ori_RAS[2]

    self.sizeX_mm = (Fin_RAS[0] - ori_RAS[0])/2.0
    self.sizeY_mm = (Fin_RAS[1] - ori_RAS[1])/2.0
    self.sizeZ_mm = (Fin_RAS[2] - ori_RAS[2])/2.0

    print('origX_mm : ', self.origX_mm  , self.origY_mm  ,  self.origZ_mm)
    print('sizeX_mm : ', self.sizeX_mm  , self.sizeY_mm  ,  self.sizeZ_mm)

  def RAStoIJK(self, spacing, orig, RAStoIJK):
   
    print('ori : ', orig[0]  , orig[1]  , orig[2])
    print('space : ', spacing[0]  , spacing[1]  , spacing[2])
    print('origX_mm : ', self.origX_mm  , self.origY_mm  ,  self.origZ_mm)
    print('sizeX_mm : ', self.sizeX_mm  , self.sizeY_mm  ,  self.sizeZ_mm)
    print('origX : ', self.origX  , self.origY  ,  self.origZ)
    print('type', type(orig[0]), type(spacing[0]), type(self.origX_mm), type(self.origX))

    Ori_RAS = [self.origX_mm - self.sizeX_mm , self.origY_mm - self.sizeY_mm, self.origZ_mm - self.sizeZ_mm, 1]
    Fin_RAS = [self.origX_mm + self.sizeX_mm, self.origY_mm + self.sizeY_mm, self.origZ_mm + self.sizeZ_mm, 1]
    ori_IJK = [0,0,0,1]
    fin_IJK = [0,0,0,1]

    ori_IJK = RAStoIJK.MultiplyPoint(Ori_RAS)   
    fin_IJK = RAStoIJK.MultiplyPoint(Fin_RAS)

    print(RAStoIJK.GetElement(0,0), RAStoIJK.GetElement(0,1), RAStoIJK.GetElement(0,2), RAStoIJK.GetElement(0,3))
    print(RAStoIJK.GetElement(1,0), RAStoIJK.GetElement(1,1), RAStoIJK.GetElement(1,2), RAStoIJK.GetElement(1,3))
    print(RAStoIJK.GetElement(2,0), RAStoIJK.GetElement(2,1), RAStoIJK.GetElement(2,2), RAStoIJK.GetElement(2,3))
    print(RAStoIJK.GetElement(3,0), RAStoIJK.GetElement(3,1), RAStoIJK.GetElement(3,2), RAStoIJK.GetElement(3,3))
   
    print('Fin_IJK : ', fin_IJK[0] , fin_IJK[1] , fin_IJK[2])
    print('ori_IJK : ', ori_IJK[0] , ori_IJK[1] , ori_IJK[2])

    if ori_IJK[0] < fin_IJK[0]:
      self.origX = round(ori_IJK[0])
      self.sizeX = round(fin_IJK[0] - ori_IJK[0])
    else :
      self.origX = round(fin_IJK[0])
      self.sizeX = -round(fin_IJK[0] - ori_IJK[0])

    if ori_IJK[1] < fin_IJK[1]:
      self.origY = round(ori_IJK[1])
      self.sizeY = round(fin_IJK[1] - ori_IJK[1])
    else :
      self.origY = round(fin_IJK[1])
      self.sizeY = -round(fin_IJK[1] - ori_IJK[1])

    if ori_IJK[2] < fin_IJK[2]:
      self.origZ = round(ori_IJK[2])
      self.sizeZ = round(fin_IJK[2] - ori_IJK[2])
    else :
      self.origZ = round(fin_IJK[2])
      self.sizeZ = -round(fin_IJK[2] - ori_IJK[2])

    print(self.origX,  self.sizeX ,self.origY, self.sizeY, self.origZ, self.sizeZ)

  def resetTodefault(self):
    self.node.setPosition(0,0,0) 
    self.node.setSize(10,10,10)

class SegModule(ScriptedLoadableModule):
  """Uses ScriptedLoadableModule base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def __init__(self, parent):
    ScriptedLoadableModule.__init__(self, parent)
    self.parent.title = "SILA3D" # TODO make this more human readable by adding spaces
    self.parent.categories = ["SILA3D - Segmentation"]
    self.parent.dependencies = []
    self.parent.contributors = ["Barthelemy Serres (ILIAD3, Universit de Tours)", "Gaetan Galisot(LIFAT, Universit de Tours)"] # replace with "Firstname Lastname (Organization)"
    self.parent.helpText = """
This is the SILA3D python scripted module for slicer3D.
It performs a step by step segmentation on the input volume using an atlas, through a web service.
"""
    self.parent.helpText += self.getDefaultModuleDocumentationLink()
    self.parent.acknowledgementText = """
SILA3D : Segmentation Interactive using a Local Atlas
""" # replace with organization, grant and thanks.

    # M.D.
    self.baseLogic = None

#
# TemplateKeyWidget
#

class SegModuleWidget(ScriptedLoadableModuleWidget):
  """Uses ScriptedLoadableModuleWidget base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setup(self):
    self.developerMode = False
  
    ScriptedLoadableModuleWidget.setup(self)
    self.logic = SegModuleLogic(slicer.modules.SegModuleInstance.baseLogic, self)

    # Instantiate and connect widgets ...
    # Segmentation tab
    
    self.sessionManagementCollapsibleButton = ctk.ctkCollapsibleButton()
    self.sessionManagementCollapsibleButton.text = "Session Management"
    self.layout.addWidget(self.sessionManagementCollapsibleButton)

    vLayout_session = qt.QVBoxLayout(self.sessionManagementCollapsibleButton)
        
    # Layout within the dummy collapsible button
    sessionManagementPanelHLayout_1 = qt.QHBoxLayout()
    vLayout_session.addLayout(sessionManagementPanelHLayout_1)
    sessionManagementPanelHLayout_2 = qt.QHBoxLayout()
    vLayout_session.addLayout(sessionManagementPanelHLayout_2)
    sessionManagementPanelHLayout_3 = qt.QHBoxLayout()
    vLayout_session.addLayout(sessionManagementPanelHLayout_3)
    
    self.sessionManagementHelpTextLbl = qt.QLabel()
    self.sessionManagementHelpTextLbl.text = "Load, save and delete new/existing session. (A session is a previous segmentation)"
    
    sessionManagementPanelHLayout_1.addWidget(self.sessionManagementHelpTextLbl)
    
    self.segSessionLbl = qt.QLabel()
    self.segSessionLbl.text = "Existing sessions:"
    self.segSessionLbl.setStyleSheet("""
        min-width: 25px;
    """)
    self.sessionSelector = qt.QComboBox()
    self.sessionSelector.setMinimumWidth(190)
    self.loadSessionBtn = qt.QPushButton("Load")
    self.loadSessionBtn.setStyleSheet("""
        height: 10px;
        margin-left: 4px;
        width: 25px;
    """)
    self.loadSessionBtn.connect('clicked(bool)', self.loadSessionBtnClicked)
    self.deleteSessionBtn = qt.QPushButton("Delete")
    self.deleteSessionBtn.setStyleSheet("""
        height: 10px;
        margin-left: 4px;
        width: 25px;
    """)
    self.deleteSessionBtn.connect('clicked(bool)', self.deleteSession)
    self.downloadSessBtn = qt.QPushButton("Download")
    self.downloadSessBtn.setStyleSheet("""
        height: 10px;
        margin-left: 4px;
        width: 25px;
    """)
    self.downloadSessBtn.connect('clicked(bool)', self.downloadSession)
    
    sessionManagementPanelHLayout_2.addWidget(self.segSessionLbl)
    sessionManagementPanelHLayout_2.addWidget(self.sessionSelector)
    sessionManagementPanelHLayout_2_v = qt.QVBoxLayout()
    sessionManagementPanelHLayout_2.addLayout(sessionManagementPanelHLayout_2_v)
    sessionManagementPanelHLayout_2_1 = qt.QHBoxLayout()
    sessionManagementPanelHLayout_2_2 = qt.QHBoxLayout()
    sessionManagementPanelHLayout_2_v.addLayout(sessionManagementPanelHLayout_2_1)
    sessionManagementPanelHLayout_2_v.addLayout(sessionManagementPanelHLayout_2_2)
    sessionManagementPanelHLayout_2_1.addWidget(self.loadSessionBtn)
    sessionManagementPanelHLayout_2_1.addWidget(self.deleteSessionBtn)
    sessionManagementPanelHLayout_2_2.addWidget(self.downloadSessBtn)
    
    self.sessionManagementHelpTextLbl1 = qt.QLabel()
    self.sessionManagementHelpTextLbl1.text = "Saving the existing session does not override a loaded session:"
    self.sessionManagementHelpTextLbl1.setStyleSheet("""
        margin-top: 10px;
    """)
    self.saveDataButton = qt.QPushButton("Save Session Data")
    self.saveDataButton.setStyleSheet("""
        width: 70px;
        height: 17px;
        margin-top: 10px;
    """)
    self.saveDataButton.connect('clicked(bool)', self.saveSession)
    
    sessionManagementPanelHLayout_3.addWidget(self.sessionManagementHelpTextLbl1)
    sessionManagementPanelHLayout_3.addWidget(self.saveDataButton)
    
    
    # self.loadDataButton = qt.QPushButton("Load Session Data")
    # self.loadDataButton.toolTip = "Load previous results from last saved session"
    # self.loadDataButton.setStyleSheet("""
        # min-width: 80px;
        # height: 17px;
    # """)
    # self.loadDataButton.connect('clicked(bool)', self.onLoadData)
    # sessionManagementPanelHLayout_3.addWidget(self.loadDataButton)

    # self.saveDataButton = qt.QPushButton("Save Session Data")
    # self.saveDataButton.toolTip = "Save current results"
    # self.saveDataButton.setStyleSheet("""
        # min-width: 80px;
        # height: 17px;
    # """)
    # self.saveDataButton.connect('clicked(bool)', self.onSaveData)
    # sessionManagementPanelHLayout_3.addWidget(self.saveDataButton)
    
    #############################################################
    # Algorithm Configuration Area
    #
    self.configurationCollapsibleButton = ctk.ctkCollapsibleButton()
    self.configurationCollapsibleButton.text = "Segmentation configuration"
    self.layout.addWidget(self.configurationCollapsibleButton)
    
    vLayout_config = qt.QVBoxLayout(self.configurationCollapsibleButton)
    
    # Layout within the dummy collapsible button
    dataConfigFormLayout_1 = qt.QHBoxLayout()
    vLayout_config.addLayout(dataConfigFormLayout_1)
    dataConfigFormLayout_2 = qt.QHBoxLayout()
    vLayout_config.addLayout(dataConfigFormLayout_2)
    
    self.regionLbl = qt.QLabel()
    self.regionLbl.text = "Atlas region:"
    self.regionLbl.setStyleSheet("""
        min-width: 25px;
    """)
    self.regionSelector = qt.QComboBox()
    self.regionSelector.currentIndexChanged.connect(self.onRegionSelectionChange)
    self.regionSelector.setMinimumWidth(170)
    self.ROIInitButton = qt.QPushButton("Setup ROI")
    self.ROIInitButton.toolTip = "ROI initialization"
    self.ROIInitButton.setStyleSheet("""
        height: 15px;
        margin-left: 4px;
        width: 70px;
    """)
    self.ROIInitButton.connect('clicked(bool)', self.onSetupROI)
    
    dataConfigFormLayout_1.addWidget(self.regionLbl)
    dataConfigFormLayout_1.addWidget(self.regionSelector)
    dataConfigFormLayout_1.addWidget(self.ROIInitButton)
    
    # push all controls to the right
    dataConfigFormLayout_2.setContentsMargins(0, 10, 0, 0)
    dataConfigFormLayout_2.addStretch()
    # processing msg
    self.msgProcessingLbl = qt.QLabel()
    self.msgProcessingLbl.text = "This task may take several minutes!"
    self.msgProcessingLbl.setStyleSheet("""
        color: #013387;
    """)
    self.msgProcessingLbl.hide()
    dataConfigFormLayout_2.addWidget(self.msgProcessingLbl)
    # processing progress
    self.percentageProcessingLbl = qt.QLabel()
    self.percentageProcessingLbl.text = "0%"
    self.percentageProcessingLbl.setStyleSheet("""
        color: #013387;
    """)
    self.percentageProcessingLbl.hide()
    dataConfigFormLayout_2.addWidget(self.percentageProcessingLbl)
    # Gif
    self.loadingGifLbl = qt.QLabel()
    self.loadingGifLbl.setFixedWidth(12)
    self.loadingGifLbl.setFixedHeight(12)
    moduleDir = os.path.dirname(slicer.util.modulePath(self.__module__))
    loadingiconPath = os.path.join(moduleDir, 'Resources/Imgs', 'ajax-loader.gif')
    self.loadingGifMovie = qt.QMovie(loadingiconPath);
    self.loadingGifMovie.setScaledSize(qt.QSize(12, 12))
    self.loadingGifLbl.setMovie(self.loadingGifMovie)
    self.loadingGifLbl.hide()
    self.loadingGifMovie.stop()
    dataConfigFormLayout_2.addWidget(self.loadingGifLbl)
    
    self.applyButton = qt.QPushButton("Run segmentation")
    self.applyButton.setStyleSheet("""
        min-width: 80px;
        height: 17px;
    """)
    self.applyButton.connect('clicked(bool)', self.onApplyButton)
    dataConfigFormLayout_2.addWidget(self.applyButton)
    
    #######################################
    # Results Management
    #
    self.resultsDataManagementCollapsibleButton = ctk.ctkCollapsibleButton()
    self.resultsDataManagementCollapsibleButton.text = "Results Data Management"
    self.layout.addWidget(self.resultsDataManagementCollapsibleButton)

    vLayout_res = qt.QVBoxLayout(self.resultsDataManagementCollapsibleButton)
    
    # Layout within the dummy collapsible button
    dataResFormLayout_1 = qt.QHBoxLayout()
    vLayout_res.addLayout(dataResFormLayout_1)
    dataResFormLayout_2 = qt.QHBoxLayout()
    vLayout_res.addLayout(dataResFormLayout_2)
    
    self.segRegionsLbl = qt.QLabel()
    self.segRegionsLbl.text = "Segmented regions:"
    self.segRegionsLbl.setStyleSheet("""
        min-width: 25px;
    """)
    self.segmentedRegionSelector = qt.QComboBox()
    self.segmentedRegionSelector.setMinimumWidth(170)
    self.segmentedRegionSelector.currentIndexChanged.connect(
        self.onSegmentedRegionSelectionChange)
    self.removeRegionButton = qt.QPushButton("Remove")
    self.removeRegionButton.toolTip = "Remove segmentation for selected region"
    self.removeRegionButton.setStyleSheet("""
        height: 15px;
        margin-left: 4px;
        width: 70px;
    """)
    self.removeRegionButton.connect('clicked(bool)', self.onRemoveRegion)
    
    dataResFormLayout_1.addWidget(self.segRegionsLbl)
    dataResFormLayout_1.addWidget(self.segmentedRegionSelector)
    dataResFormLayout_1.addWidget(self.removeRegionButton)
    
    self.segUpdateLbl = qt.QLabel()
    self.segUpdateLbl.text = "Update Segmentation (volume):"
    self.segUpdateLbl.setStyleSheet("""
        min-width: 25px;
        margin-top: 10px;
    """)
    self.dlSegVolDataButton = qt.QPushButton("Update")
    self.dlSegVolDataButton.setStyleSheet("""
        min-width: 168px;
        height: 17px;
        margin-top: 10px;
    """)
    self.dlSegVolDataButton.connect('clicked(bool)', self.onDownloadSegVolResults)
    
    dataResFormLayout_2.addWidget(self.segUpdateLbl)
    dataResFormLayout_2.addStretch()
    dataResFormLayout_2.addWidget(self.dlSegVolDataButton)
    
    #######################################
    # Upload custom session to server
    #
    self.customSessionCollapsibleButton = ctk.ctkCollapsibleButton()
    self.customSessionCollapsibleButton.text = "Upload custom segmentation"
    self.layout.addWidget(self.customSessionCollapsibleButton)

    vLayout_cust = qt.QVBoxLayout(self.customSessionCollapsibleButton)
    
    customSessionFormLayout_1 = qt.QHBoxLayout()
    vLayout_cust.addLayout(customSessionFormLayout_1)
    customSessionFormLayout_2 = qt.QHBoxLayout()
    vLayout_cust.addLayout(customSessionFormLayout_2)
    
    self.customSessionHelpTextLbl = qt.QLabel()
    self.customSessionHelpTextLbl.text = "Click on this button to upload current segmentation to the server and save it in a session.<br/>Convert manual modifications to a session and save it on the server."
    customSessionFormLayout_1.addWidget(self.customSessionHelpTextLbl)
    
    self.customSessionBtn = qt.QPushButton("Upload && save")
    self.customSessionBtn.setStyleSheet("""
        min-width: 168px;
        height: 17px;
        margin-top: 10px;
    """)
    self.customSessionBtn.connect('clicked(bool)', self.uploadAndSaveCustomSession)
    
    customSessionFormLayout_2.addStretch()
    customSessionFormLayout_2.addWidget(self.customSessionBtn)
    
    # Add vertical spacer
    self.layout.addStretch(1)
    
    # Start with all disabled
    self.toggleUIControls(False)
    
    # Fill the dropdown lists
    self.initUIForSegModule()
  
  def delayDisplay(self,message,msec=100):
    """This utility method displays a small dialog and waits.
    This does two things: 1) it lets the event loop catch up
    to the state of the test so that rendering and widget updates
    have all taken place before the test continues and 2) it
    shows the user/developer/tester the state of the test
    so that we'll know when it breaks.
    """
    print(message)
    self.info = qt.QDialog()
    self.info.setWindowTitle('Info')
    if self.logic.baseLogic.WICON is not None:
      self.info.setWindowIcon(self.logic.baseLogic.WICON)
    self.info.setWindowFlags(qt.Qt.WindowTitleHint)
    self.info.resize(150,30)
    self.infoLayout = qt.QVBoxLayout()
    self.info.setLayout(self.infoLayout)
    self.label = qt.QLabel(message,self.info)
    self.infoLayout.addWidget(self.label, qt.Qt.AlignCenter)
    qt.QTimer.singleShot(msec, self.info.close)
    self.info.exec_()

  def onExit(self):
    slicer.mrmlScene.Clear()
    
  def initUIForSegModule(self):
    if (self.logic.baseLogic.LOGGEDIN and 
        self.logic.baseLogic.APP_RUNNING):
      
      # clearn the scene to restart over
      slicer.mrmlScene.Clear()
      
      self.delayDisplay("Initializing App. Please wait...", 1000)
      
      print("DEBUG: initUIForSegModule(" + self.logic.baseLogic.USER_ID + "," + 
        self.logic.baseLogic.CURRENT_MODEL + "," + 
        self.logic.baseLogic.CURRENT_MODEL_FLD + "," +
        str(self.logic.baseLogic.CURRENT_MODEL_PUBLIC) + "," +
        self.logic.baseLogic.CURRENT_T1 + "," +
        self.logic.baseLogic.CURRENT_T1_FILE + "," +
        str(self.logic.baseLogic.CURRENT_T1_PUBLIC) + ")")
      
      #init the app
      self.logic.initApp(self.logic.baseLogic.USER_ID,
                self.logic.baseLogic.CURRENT_MODEL_FLD, 
                self.logic.baseLogic.CURRENT_MODEL_PUBLIC,
                self.logic.baseLogic.CURRENT_T1_FILE,
                self.logic.baseLogic.CURRENT_T1_PUBLIC,
                self.logic.baseLogic.FORCE_RETRY)
      if self.logic.baseLogic.FORCE_RETRY: # to avoid getting stuck in loop
        self.logic.baseLogic.FORCE_RETRY = False
      print("DEBUG: done initApp")
      
      if (self.logic.baseLogic.CURRENT_MODEL != "NEW"):
        self.logic.initSegmentationMethod(self.logic.CURRENT_METHOD)
        print("DEBUG: done initSegmentationMethod")
        #enable segmentation configuration section
        self.configurationCollapsibleButton.enabled = True
      else:
        #disable segmentation configuration section
        self.configurationCollapsibleButton.enabled = False
      
      # Update Display
      self.logic.updateMainSceneDisplay()
      
      # Setup Available regions (populate CB with Atlas Regions)
      self.setupAvailableRegions()
      
      # Load Current User Session Data
      self.initSession(True, True, False, False)
      
      print("DEBUG: exit initUIForSegModule")
    
      self.toggleUIControls(True)
    else:
      self.toggleUIControls(False)  
  
  def toggleUIControls(self, on):
    if on:
      self.sessionManagementCollapsibleButton.enabled = True
      self.configurationCollapsibleButton.enabled = True
      self.resultsDataManagementCollapsibleButton.enabled = True
      self.customSessionCollapsibleButton.enabled = True
    else:
      self.sessionManagementCollapsibleButton.enabled = False
      self.configurationCollapsibleButton.enabled = False
      self.resultsDataManagementCollapsibleButton.enabled = False
      self.customSessionCollapsibleButton.enabled = False
  
  def initSession(self, unload, updateUI, loadLatest, onlyRefresh):
    print("DEBUG: initSession for user")
    #in case of new model .. all sessions will be returned and the user will pick
    found = self.logic.getAllPreviousSessions("")
    sessions = {}
    if found:
      sessions = self.logic.AVAILABLE_SESSIONS
    # populate them in the dropdownlist
    self.sessionSelector.clear()
    if (found and len(sessions) > 0):
      self.sessionSelector.addItem("-- New session --", 0)
      for i, v in enumerate(sessions):
        d = self.logic.AVAILABLE_SESSIONS_DATE[i]
        self.sessionSelector.addItem(v + ' (' + d + ')', i+1)
      self.loadSessionBtn.enabled = True
      self.deleteSessionBtn.enabled = True
      self.downloadSessBtn.enabled = True
      # load the selected (last) session
      # if a model is selected --> try to find last session with same model's name
      sessionToLoad = "NONE"
      idxSessionToLoad = 0
      if self.logic.baseLogic.CURRENT_MODEL != "NEW" and loadLatest:
        for i, v in enumerate(sessions):
          #print(self.logic.baseLogic.CURRENT_MODEL + " " + v + " " + str(self.logic.baseLogic.CURRENT_MODEL.lower() in v.lower()))
          if self.logic.baseLogic.CURRENT_MODEL.lower() == v.split("_")[0].lower():
            #get the first 
            sessionToLoad = v
            idxSessionToLoad = i+1
            break
      if onlyRefresh:
        self.logic.baseLogic.LOADED_SESSION_DIRNAME = sessionToLoad
        self.sessionSelector.setCurrentIndex(idxSessionToLoad)
        self.updateSegmentedRegions()
      else:
        self.loadSession(sessionToLoad, idxSessionToLoad, unload, updateUI)
    else:
      self.sessionSelector.addItem("-- No sessions --")
      self.loadSessionBtn.enabled = False
      self.deleteSessionBtn.enabled = False
      self.downloadSessBtn.enabled = False
      
      # no sessions means no segmented regions
      self.segmentedRegionSelector.clear()
      self.segmentedRegionSelector.addItem("-- No regions --")
      self.removeRegionButton.enabled = False
      self.dlSegVolDataButton.enabled = False
  
  def loadSession(self, defaultSession, defaultIdxSession, unload, updateUI):
    print("Entering loadSession (" + defaultSession + ")")
    # if defaultSession == "":
      # session = self.sessionSelector.currentText
    # else:
      # session = defaultSession
    session = defaultSession
    print("Session to load: " + session)
    if (session == "NONE"):
      # no session selected means no segmented regions
      self.segmentedRegionSelector.clear()
      self.segmentedRegionSelector.addItem("-- No regions --", -1)
      self.removeRegionButton.enabled = False
      self.dlSegVolDataButton.enabled = False
      if unload and self.logic.segmentationVolumeNode != None:
        res = self.logic.unloadSegmentation()
        if res:
          self.delayDisplay("Last session is unloaded successfully", 1500)
        else:
          self.delayDisplay("Failed to unload last session", 1500)
      if updateUI:
        # self.logic.unloadLoadedImageInViews()
        self.logic.loadEmptyIRMImage()
      self.logic.baseLogic.LOADED_SESSION_DIRNAME = ""
    else:
      self.toggleUIControls(False)
      if self.logic.loadSessionData(session):
        self.logic.baseLogic.LOADED_SESSION_DIRNAME = session
        if (defaultSession != ""):
          self.sessionSelector.setCurrentIndex(defaultIdxSession)
        self.delayDisplay("The session has been loaded.", 1500)
        
        # Update Segmented Regions lists (populate CB with segmented Atlas Regions)
        self.updateSegmentedRegions()
        
        # Download last results from server
        self.logic.downloadLabelmapVolumeData(session)
        print("DEBUG: done downloadLabelmapVolumeData")
        
      else:
        self.delayDisplay("Error: The session could not be loaded.", 1500)
      self.toggleUIControls(True)
  
  def loadSessionBtnClicked(self):
    if self.sessionSelector.currentIndex == 0:
      self.loadSession("NONE", 0, True, True)
    elif self.sessionSelector.currentIndex >= 0:
      k = self.logic.AVAILABLE_SESSIONS[self.sessionSelector.currentIndex-1]
      self.loadSession(k, self.sessionSelector.currentIndex, False, False)
  
  def deleteSession(self):
    if self.sessionSelector.currentIndex == 0:
      self.delayDisplay("No session is selected!", 1500)
      return
    session = self.logic.AVAILABLE_SESSIONS[self.sessionSelector.currentIndex-1]
    if (not slicer.util.confirmOkCancelDisplay(
            "Are you sure you want to delete session: \"" +
            str(session) + 
            "\"", windowTitle="Confirm session deletion")):
      return
    
    self.toggleUIControls(False)
    if self.logic.deleteSessionData(session):
      # if deleting the current loaded session then refresh
      if (self.logic.baseLogic.LOADED_SESSION_DIRNAME == session):
        self.logic.baseLogic.LOADED_SESSION_DIRNAME = ""
        self.initSession(True, True, False, False)
      else:
        self.sessionSelector.removeItem(self.sessionSelector.currentIndex)
        for i,v in enumerate(self.logic.AVAILABLE_SESSIONS):
          if v == session:
            idx = i
        #print('to delete', idx)
        del self.logic.AVAILABLE_SESSIONS[idx]
        del self.logic.AVAILABLE_SESSIONS_DATE[idx]
        # update session selector to loaded session
        if (self.logic.baseLogic.LOADED_SESSION_DIRNAME != ""):
          idx = -1
          #print(self.logic.AVAILABLE_SESSIONS, session, self.logic.baseLogic.LOADED_SESSION_DIRNAME)
          #print('after deletion', self.logic.AVAILABLE_SESSIONS)
          for i,v in enumerate(self.logic.AVAILABLE_SESSIONS):
            if v == self.logic.baseLogic.LOADED_SESSION_DIRNAME:
              idx = i+1
          self.sessionSelector.setCurrentIndex(idx)
        else: # update session selector to new 
          self.sessionSelector.setCurrentIndex(0)
      self.delayDisplay("The session has been safely deleted.", 1500)
    else:
      self.delayDisplay("Error: The session could not be deleted.", 1500)
    self.toggleUIControls(True)
    
  def downloadSession(self):
    if self.sessionSelector.currentIndex == 0:
      self.delayDisplay("No session is selected!", 1500)
      return
    session = self.logic.AVAILABLE_SESSIONS[self.sessionSelector.currentIndex-1]
    # open dialog and ask for location
    options = qt.QFileDialog.Options()
    options |= qt.QFileDialog.DontUseNativeDialog
    options |= qt.QFileDialog.DontUseCustomDirectoryIcons
    dialog = qt.QFileDialog(None, 'Open folder')
    dialog.setOptions(options)
    dialog.setFileMode(qt.QFileDialog.DirectoryOnly)
    dialog.setAcceptMode(qt.QFileDialog.AcceptSave)
    if dialog.exec_() == qt.QDialog.Accepted:
      path = dialog.selectedFiles()[0]
      if self.logic.downloadSessionAndSaveToFile(session, path):
        self.delayDisplay("Session has been successfully downloaded", 1500)
      else:
        self.delayDisplay("An error has occured and prevented from downloading the session. Please contact support team.")
    else: #no folder has been selected
      pass
  
  def saveSession(self):
    print("DEBUG: saveSession")
    # check if there are no region segmented then stop saving
    # if (self.segmentedRegionSelector.count == 1 and 
        # self.segmentedRegionSelector.currentData == -1):
    if not self.logic.baseLogic.DATA_NEED_SAVE:
      self.delayDisplay("Nothing has been done so far to save!", 1500)
    else:
      self.toggleUIControls(False)
      if self.logic.saveSessionData():
        self.logic.baseLogic.DATA_NEED_SAVE = False
        self.delayDisplay("The current session has been saved!", 1500)
        # redo an initSession to refresh the list of sessions
        self.initSession(False, False, True, True)
      else:
        self.delayDisplay("Error: The current session could not be saved!", 1500)
      self.toggleUIControls(True)
  
  def onRegionSelectionChange(self, i):
    it = self.regionSelector.itemData(i)
    if (it <= - 1):
      pass
    else:
      self.logic.CURRENT_REGION_ID = it
      print("DEBUG: Selected Region ID: " + str(self.logic.CURRENT_REGION_ID))
      self.logic.setupRegion(self.logic.CURRENT_REGION_ID)
  
  def onSetupROI(self):
    print("DEBUG: onSetupROI")
    if self.logic.CURRENT_REGION_ID <= -1:
      self.delayDisplay("A region must be selected to setup ROI!", 1500)
      return
    self.logic.setupROI(self.logic.CURRENT_REGION_ID)
  
  def onApplyButton(self):
    if self.logic.baseLogic.CURRENT_MODEL == 'NEW':
      self.delayDisplay("There is no selected model. Can't start segmentation!", 1500)
      return
    #get bounding box position from current position
    try:
      self.logic.setupBoundingBox()
    except:
      self.delayDisplay("ROI is not set!", 1500)
      return
    if (slicer.util.confirmOkCancelDisplay(
            "This operation may take few minutes. Do you wish to proceed?", 
            windowTitle="Confirm starting segmentation")):
      self.startSegmentation()
     
     # if sucessful segmentation
     # if (self.logic.startSegmentation()):
      # # Update Segmented Regions lists (populate CB with segmented Atlas Regions)
      # self.updateSegmentedRegions()

      # # Download last results from server
      # self.logic.downloadLabelmapVolumeData()

      # # self.resultsDataManagementCollapsibleButton.enabled = True
      # # self.resultsDataManagementCollapsibleButton.collapsed = False  
  
  def startSegmentation(self):
    # if segmentation has started sucessfully
    if self.logic.startSegmentationNew():
      #start the timer and keep calling the check api to ge the status
      self.logic.baseLogic.SEG_TIMER.timeout.connect(self.checkSegmentationStatus)
      self.logic.baseLogic.SEG_TIMER.start(30000)
      print("Segmentation timer has started")
      self.toggleUIControls(False)
      # show icon loading
      self.showHideLoading(True, 0)
      
      self.delayDisplay("Segmentation has started!", 1500)
    else:
      self.delayDisplay("Segmentation could not start. Please try again!", 1500)
  
  def checkSegmentationStatus(self):
    # call the api
    success, done, msg = self.logic.checkSegmentationStatus()
    if not success:
      self.showHideLoading(False, 0)
      self.endSegmentation(False, msg)
    if done:
      if self.logic.baseLogic.SEG_TIMER != None and self.logic.baseLogic.SEG_TIMER.isActive():
        self.logic.baseLogic.SEG_TIMER.stop()
        self.logic.baseLogic.SEG_TIMER = qt.QTimer()
        print("Segmentation timer is stopped")
        self.logic.baseLogic.SEG_CNT_TIMER_RAN = 100
        self.showHideLoading(True, 100)
        self.endSegmentation(success, msg)
    else: # not done so increase the counter per call
      if self.logic.baseLogic.SEG_CNT_TIMER_RAN == 90: # stop increasing and wait
        pass
      else:
        self.logic.baseLogic.SEG_CNT_TIMER_RAN = self.logic.baseLogic.SEG_CNT_TIMER_RAN + 10
        self.showHideLoading(True, self.logic.baseLogic.SEG_CNT_TIMER_RAN)
  
  def endSegmentation(self, success, msg):
    print("Segmentation is done.")
    
    if not success:
      self.delayDisplay("Failed to perform segmentation.\ndetails: " + msg, 1500)
      self.logic.baseLogic.resetSegModuleGlobalParams()
      self.logic.baseLogic.exitProc(False)
      self.logic.baseLogic.forceModuleReload("DataModule")
      self.logic.baseLogic.forceModuleReload("SegModule")
      self.logic.baseLogic.forceModuleReload("ModelModule")
      self.logic.baseLogic.switchToTab(slicer, "DataModule")
    else:
      # Update Segmented Regions lists (populate CB with segmented Atlas Regions)
      self.updateSegmentedRegions()

      # Download last results from server
      self.logic.downloadLabelmapVolumeData("tmp")
    
      self.delayDisplay("Segmentation has been done!", 1500)
      
      #force save before exit
      self.logic.baseLogic.DATA_NEED_SAVE = True
    
    self.showHideLoading(False, 0)
    self.logic.baseLogic.SEG_CNT_TIMER_RAN = 0
    
    # unlock the UI
    self.toggleUIControls(True)
  
  def updateSegmentedRegions(self):
    regions = self.logic.listSegmentedRegions()
    self.segmentedRegionSelector.clear()
    if len(regions) <= 0:
      self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION = []
      self.segmentedRegionSelector.addItem("-- No regions --", -1)
      self.removeRegionButton.enabled = False
      self.dlSegVolDataButton.enabled = False
    else:
      self.logic.baseLogic.SEGMENTED_REGION_LOADED_SESSION = regions.keys()
      for regionID in regions.keys():
        if (regions[regionID] == ""):
          self.segmentedRegionSelector.addItem("ID " + str(regionID), 
            regionID)
        else:
          self.segmentedRegionSelector.addItem(regions[regionID] + " (" + str(regionID) +")", 
              regionID)
      self.removeRegionButton.enabled = True
      self.dlSegVolDataButton.enabled = True
    
  def setupAvailableRegions(self):
    if (self.logic.baseLogic.CURRENT_MODEL == "NEW"):
      self.regionSelector.addItem("-- No regions --", -1)
      self.logic.baseLogic.REGION_DICTIONARY.clear()
    else:
      regions = self.logic.listRegions()
      self.regionSelector.clear()
    
      for id,name in regions.items():
        self.regionSelector.addItem(name + " (" + str(id) + ")", id)
      
      self.logic.baseLogic.REGION_DICTIONARY = regions
  
  def onSegmentedRegionSelectionChange(self, i):
    it = self.segmentedRegionSelector.itemData(i)
    if (it <= -1):
      pass
    else:
      self.logic.SEGMENTED_REGION_ID = it
      print("DEBUG: Selected Segmented Region ID: " + str(self.logic.SEGMENTED_REGION_ID))
  
  def onRemoveRegion(self):
    print ("DEBUG: onRemoveRegion - region to be removed: " + 
            str(self.logic.SEGMENTED_REGION_ID))
    if self.logic.removeSegmentedRegion(self.logic.SEGMENTED_REGION_ID):
      self.logic.baseLogic.DATA_NEED_SAVE = True
      self.updateSegmentedRegions()
      self.delayDisplay("Removing region...", 1500)
      self.logic.downloadLabelmapVolumeData("tmp")
    else:
      slicer.util.errorDisplay("Error: Region Not Removed!")
      
  def onDownloadSegVolResults(self):
    print("DEBUG: onDownloadResults")
    self.logic.downloadLabelmapVolumeData("tmp")
    
  def showHideLoading(self, show, perc):
    if show:
      self.loadingGifMovie.start()
      self.loadingGifLbl.show()
      self.percentageProcessingLbl.text = str(perc) + "%"
      self.percentageProcessingLbl.show()
      self.msgProcessingLbl.show()
    else:
      self.loadingGifMovie.stop()
      self.loadingGifLbl.hide()
      self.percentageProcessingLbl.text = ""
      self.percentageProcessingLbl.hide()
      self.msgProcessingLbl.hide()
   
  def uploadAndSaveCustomSession(self):
    res = self.logic.uploadSaveCustomSession()
    if res:
      # get session without downloading the image
      self.initSession(False, False, True, True)
      self.delayDisplay("New results have been successfully saved into a new session", 1500)
    else:
      self.delayDisplay("Failed to upload current segmentation and convert it to a session!\nPlease try again.", 1500)
   
#
# TestModuleLogic
#

class SegModuleLogic(ScriptedLoadableModuleLogic):
  """This class should implement all the actual
  computation done by your module.  The interface
  should be such that other python code can import
  this class and make use of the functionality without
  requiring an instance of the Widget.
  Uses ScriptedLoadableModuleLogic base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """
  
  # constructor
  def __init__(self, baseLogic, instanceGUI):
    self.baseLogic = baseLogic
    self.instanceModuleGUI = instanceGUI
    self.baseLogic.SEG_TIMER = qt.QTimer()
    self.baseLogic.SEG_CNT_TIMER_RAN = 0
    
    self.CURRENT_REGION_ID = -1
    self.CURRENT_METHOD = "Local"
    self.CURRENT_ROI = BBox(0, 0, 0, 10, 10, 10)
    self.segmentationVolumeNode = None
    self.T1_VolumeNode = None
    self.labelmapVolumeNode = None
    self.SEGMENTED_REGION_ID = -1
    self.AVAILABLE_SESSIONS = []
    self.AVAILABLE_SESSIONS_DATE = []
  
  def loadSessionData(self, sessionName):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/results/load/" + sessionName)
    print(req)
    ret = requests.get(req)
    sessionLoaded = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        sessionLoaded = True
    
    return sessionLoaded
  
  def deleteSessionData(self, sessionName):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/results/delete/" + sessionName)
    print(req)
    ret = requests.get(req)
    done = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        done = True
    
    return done
  
  def saveSessionData(self):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/results/save/" + self.baseLogic.CURRENT_MODEL + "/?currSess=" +
            self.baseLogic.LOADED_SESSION_DIRNAME)
    print(req)
    ret = requests.get(req)
    sessionSaved = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        sessionSaved = True
    else:
      [msg, retry] = self.baseLogic.hanldeFatalError(ret)
      self.instanceModuleGUI.delayDisplay(msg, 4000)
      if retry:
        self.baseLogic.FORCE_RETRY = True
        # some global variables to reset
        self.SEG_TIMER = None
        self.SEG_CNT_TIMER_RAN = 0
        self.baseLogic.forceModuleReload("SegModule")
        self.baseLogic.forceModuleReload("ModelModule")
        self.baseLogic.switchToTab(slicer, "SegModule")

    return sessionSaved
  
  def setupRegion(self, ID):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/region/?id=" + 
            str(ID))
    print(req)
    ret = requests.get(req)
    regionIDChanged = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        regionIDChanged = True

      slicer.util.showStatusMessage(resp["message"])

    return regionIDChanged
  
  def setupROI(self, roiID):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/bbox/auto")
    print(req)
    ret = requests.get(req)
    #bbox = []
    print(ret)
    resp = ret.json()
    print(resp)
    if ret.status_code == 200:
      try:
        if (self.CURRENT_ROI.node != None):
          slicer.mrmlScene.RemoveNode(self.CURRENT_ROI.node)
    
        self.CURRENT_ROI.node = slicer.vtkMRMLAnnotationROINode()
        self.CURRENT_ROI.node.Initialize(slicer.mrmlScene)
        
        if (resp["status"] == "OK") | (resp["status"] == "WARN"): # | (resp["Status"] == "OK"):
          # Debug
          print(resp["data"]["boundingBox"])
          bbox = resp["data"]["boundingBox"]
          self.CURRENT_ROI.setPosition(int(bbox["origin"][0]), int(bbox["origin"][1]), int(bbox["origin"][2]))
          self.CURRENT_ROI.setSize(int(bbox["dim"][0]), int(bbox["dim"][1]), int(bbox["dim"][2]))
        else:
          msgText=resp["message"]
          # slicer.util.showStatusMessage(msgText)
          # slicer.util.errorDisplay(msgText)
          self.CURRENT_ROI.resetToDefault()
        
        # T1 image is self.segmentationVol (get spacing and origin in this data structure)
        spacing = self.T1_VolumeNode.GetSpacing()
        orig = self.T1_VolumeNode.GetOrigin()
        # spacing = [0.3, 0.3, 0.3]
        # orig = [-86.0, -132.0, -32.0]
        IJKToRAS = vtk.vtkMatrix4x4()
        self.T1_VolumeNode.GetIJKToRASMatrix(IJKToRAS)
        self.CURRENT_ROI.compute(spacing, orig,IJKToRAS)
      except:
        msgText="Error retrieving boundingbox response data from server!"
        #slicer.util.errorDisplay(msgText, windowTitle="Erro message", parent=self)
        self.instanceModuleGUI.delayDisplay(msgText, 1500)
    else:
      if (self.CURRENT_ROI.node != None):
        slicer.mrmlScene.RemoveNode(self.CURRENT_ROI.node)
      [msg, retry] = self.baseLogic.hanldeFatalError(ret)
      self.instanceModuleGUI.delayDisplay(msg, 4000)
      if retry:
        self.baseLogic.FORCE_RETRY = True
        # some global variables to reset
        self.SEG_TIMER = None
        self.SEG_CNT_TIMER_RAN = 0
        self.baseLogic.forceModuleReload("SegModule")
        self.baseLogic.forceModuleReload("ModelModule")
        self.baseLogic.switchToTab(slicer, "SegModule")
    
  def startSegmentation(self):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/start")
    print(req)
    ret = requests.get(req, timeout=None)
    processDone = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        processDone = True
        self.instanceModuleGUI.delayDisplay('Segmentation Finished!', 1000)

    return processDone

  def listSegmentedRegions(self):
    segRegionDic = {}
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/segmented/")
    ret = requests.get(req)
    initialized = False
    print(req,ret)
    if ret.status_code == 200:
      resp = ret.json()
      #print resp
      if resp["status"] == "OK":
        print(resp["data"])
        regions = resp["data"]
        
        #print "regions="+regions
        for region in regions:
          if (int(region["id"]) in self.baseLogic.REGION_DICTIONARY.keys()):
            segRegionDic[int(region["id"])] = self.baseLogic.REGION_DICTIONARY[int(region["id"])]
          else:
            segRegionDic[int(region["id"])] = ""

      segRegionDicOrd = collections.OrderedDict(sorted(segRegionDic.items()))
      print(segRegionDic)
      print(segRegionDicOrd)

    return segRegionDicOrd
  
  def updateMainSceneDisplay(self):
    #imgURL = self.getCurrentImageURL(self.baseLogic.CURRENT_T1_FILE,self.baseLogic.CURRENT_T1_PUBLIC)
    imgURL = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
              "/neurobrainweb/data/image/" + self.baseLogic.CURRENT_T1_FILE + '?userID=' + self.baseLogic.USER_ID + 
              "&isPublic=" + str(self.baseLogic.CURRENT_T1_PUBLIC))
    print('Downloading url: '+imgURL)
    downloads = (
        (imgURL, self.baseLogic.CURRENT_T1_FILE, slicer.util.loadVolume),
        )

    for url,name,loader in downloads:
      filePath = slicer.app.temporaryPath + '/' + name
      #if not os.path.exists(filePath) or os.stat(filePath).st_size == 0:
      #  logging.info('Requesting download %s from %s...\n' % (name, url))
      #  urlretrieve(url, filePath)
      if os.path.exists(filePath):
        os.remove(filePath)
      urlretrieve(url, filePath)
      if loader:
        logging.info('Loading %s...' % (name,))
        print("filePath="+filePath)
        [success, loadedVolumeNode] = loader(filePath, returnNode=True)
        self.T1_VolumeNode = loadedVolumeNode # slicer.util.getNode(pattern=name)
        
        layoutManager = slicer.app.layoutManager()
        for sliceViewName in layoutManager.sliceViewNames():
          controller = layoutManager.sliceWidget(sliceViewName).sliceController()
          controller.setSliceVisible(True)

        slicer.util.resetThreeDViews()
    
  
  def getAllPreviousSessions(self, model):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/results/getAll?model=" + 
            model)
    print(req)
    ret = requests.get(req)
    found = False
    self.AVAILABLE_SESSIONS = []
    self.AVAILABLE_SESSIONS_DATE = []
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      if resp["status"] == "OK":
        found = True
        for e in resp['sessions']:
          print(e)
          self.AVAILABLE_SESSIONS.append(e["name"])
          self.AVAILABLE_SESSIONS_DATE.append(e["lastModified"])
    
    return found
  
  def downloadLabelmapVolumeData(self, session):
    currentSession = session # self.baseLogic.LOADED_SESSION_DIRNAME
    
    # if there is a loaded session then we can download the files
    if currentSession == "":
      self.instanceModuleGUI.delayDisplay("No previous session is found.", 1500)
      return
    
    resultImgURL = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
        "/neurobrainweb/" + self.baseLogic.USER_ID + "/results/download/" + currentSession)
    downloads = (
          (resultImgURL,  "results.nii.gz",    slicer.util.loadLabelVolume),
        )
    
    for url,name,loader in downloads:
      filePath = slicer.app.temporaryPath + '/' + name
      if os.path.exists(filePath):
        logging.info("previous file exits: deleting..")
        os.remove(filePath)
        logging.info("previous file exits: deleted.")
      
      logging.info('Requesting download %s from %s\n' % (name, url))
      f, headers = urlretrieve(url, filePath)
      if (headers["Content-Type"] == "application/json"): # this means download has failed
        self.instanceModuleGUI.delayDisplay("Failed to download session files!", 1500)
        return
      if loader:
        print("Loading filePath= " + filePath)
        if self.segmentationVolumeNode != None:
          slicer.mrmlScene.RemoveNode(self.segmentationVolumeNode)
        if self.labelmapVolumeNode != None:
          slicer.mrmlScene.RemoveNode(self.labelmapVolumeNode)
        self.segmentationVolumeNode = slicer.mrmlScene.AddNewNodeByClass('vtkMRMLSegmentationNode', 'SegmentationLabelmap')
        [success, allSegmentsLabelmapNode] = loader(filePath, returnNode=True)
        print("success: " + str(success))
        msg='%s Loaded.' % (name,)
        slicer.util.showStatusMessage(msg)
        
        # workaround to fix bug: when image has one region, slicer shows 'AllSegmentsLabelmap' instead of regionID
        # sol: check if image has only one region, then name the labelmap with region ID
        img_nii = nib.load(filePath)
        regionIDs = np.unique(img_nii.get_fdata())
        if len(regionIDs) == 2: # ex. [0. 40.]
          regionID = int(regionIDs[regionIDs != 0][0])
          allSegmentsLabelmapNode.SetName(str(regionID))
        else:
          allSegmentsLabelmapNode.SetName('AllSegmentsLabelmap')
        
        self.labelmapVolumeNode = allSegmentsLabelmapNode

        #segNode = slicer.mrmlScene.AddNewNodeByClass("vtkMRMLSegmentationNode")
        slicer.vtkSlicerSegmentationsModuleLogic.ImportLabelmapToSegmentationNode(self.labelmapVolumeNode, self.segmentationVolumeNode)

        layoutManager = slicer.app.layoutManager()
        for sliceViewName in layoutManager.sliceViewNames():
          controller = layoutManager.sliceWidget(sliceViewName).sliceController()
          controller.setSliceVisible(True)

        slicer.util.resetThreeDViews()
    
    self.instanceModuleGUI.delayDisplay('Finished with download and loading!', 1500)
  
  def downloadSessionAndSaveToFile(self, currentSession, fileDest):
    sessSegURL = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
        "/neurobrainweb/" + self.baseLogic.USER_ID + "/results/download/" + currentSession)
    
    fileDest = os.path.join(fileDest, currentSession + '.nii.gz')
    if os.path.exists(fileDest):
      logging.info("previous file exits: deleting..")
      os.remove(fileDest)
      logging.info("previous file exits: deleted.")
      
    logging.info('Requesting download from %s\n' % (sessSegURL))
    f, headers = urlretrieve(sessSegURL, fileDest)
    if (headers["Content-Type"] == "application/json"): # this means download has failed
      self.instanceModuleGUI.delayDisplay("Failed to download session files!", 1500)
      return False
    return True
  
  def unloadLoadedImageInViews(self):
    if self.segmentationVolumeNode != None:
      slicer.mrmlScene.RemoveNode(self.segmentationVolumeNode)
      self.segmentationVolumeNode = None
    if self.labelmapVolumeNode != None:
      slicer.mrmlScene.RemoveNode(self.labelmapVolumeNode)
      self.labelmapVolumeNode = None
    layoutManager = slicer.app.layoutManager()
    for sliceViewName in layoutManager.sliceViewNames():
      controller = layoutManager.sliceWidget(sliceViewName).sliceController()
      controller.setSliceVisible(True)
    slicer.util.resetThreeDViews()
  
  def loadEmptyIRMImage(self):
    file_input_irm = slicer.app.temporaryPath + '/' + self.baseLogic.CURRENT_T1_FILE
    if not os.path.exists(file_input_irm):
      self.instanceModuleGUI.delayDisplay('No image loaded. Failed to start a new session', 1500)
      return
    
    # create an empty version of irm image as label image
    nib.Nifti1Header.quaternion_threshold = -1e-06 # to solve issue https://github.com/nipy/nibabel/issues/626
    img_irm = nib.load(file_input_irm)
    data = img_irm.get_fdata() * 0
    
    name = 'results.nii.gz'
    file_output_nii = slicer.app.temporaryPath + '/' + name
    img_nii = nib.Nifti1Image(data, img_irm.affine, img_irm.header)
    nib.save(img_nii, file_output_nii)
    # load empty image to volume
    loader = slicer.util.loadLabelVolume
    if loader:
      print("Loading filePath= " + file_output_nii)
      if self.segmentationVolumeNode != None:
        slicer.mrmlScene.RemoveNode(self.segmentationVolumeNode)
      if self.labelmapVolumeNode != None:
        slicer.mrmlScene.RemoveNode(self.labelmapVolumeNode)
      self.segmentationVolumeNode = slicer.mrmlScene.AddNewNodeByClass('vtkMRMLSegmentationNode', 'SegmentationLabelmap')
      [success, allSegmentsLabelmapNode] = loader(file_output_nii, returnNode=True)
      print("success: " + str(success))
      msg='%s Loaded.' % (name,)
      slicer.util.showStatusMessage(msg)
      
      allSegmentsLabelmapNode.SetName('AllSegmentsLabelmap')
      
      self.labelmapVolumeNode = allSegmentsLabelmapNode
      
      #segNode = slicer.mrmlScene.AddNewNodeByClass("vtkMRMLSegmentationNode")
      slicer.vtkSlicerSegmentationsModuleLogic.ImportLabelmapToSegmentationNode(self.labelmapVolumeNode, self.segmentationVolumeNode)
      
      layoutManager = slicer.app.layoutManager()
      for sliceViewName in layoutManager.sliceViewNames():
        controller = layoutManager.sliceWidget(sliceViewName).sliceController()
        controller.setSliceVisible(True)
      
      slicer.util.resetThreeDViews()
  
  def setupBoundingBox(self):
    #lim = [0,0,0,0,0,0]
    #self.CURRENT_ROI.node.GetBounds(lim)
    #print 'lin : ', lim[0],lim[1],lim[2],lim[3],lim[4],lim[5]
    posi = [0,0,0]
    self.CURRENT_ROI.node.GetXYZ(posi)
    print('posi : ', posi[0],posi[1],posi[2])
    tail = [0,0,0]
    self.CURRENT_ROI.node.GetRadiusXYZ(tail)
    print('tail : ', tail[0],tail[1],tail[2])


    #print 'lin : ', lim[0],lim[1],lim[2],lim[3],lim[4],lim[5]
    #self.CURRENT_ROI.setPosition_mm(lim[0],lim[2],lim[4])
    #self.CURRENT_ROI.setSize_mm(lim[1]-lim[0],lim[3]-lim[2],lim[5]-lim[4])

    self.CURRENT_ROI.setPosition_mm(posi[0],posi[1],posi[2])
    self.CURRENT_ROI.setSize_mm(tail[0],tail[1],tail[2])

    spacing = self.T1_VolumeNode.GetSpacing()
    orig = self.T1_VolumeNode.GetOrigin()
    RAStoIJK = vtk.vtkMatrix4x4()
    self.T1_VolumeNode.GetRASToIJKMatrix(RAStoIJK)
    self.CURRENT_ROI.RAStoIJK(spacing,orig,RAStoIJK)
    
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/bbox/?x=" + 
            str(self.CURRENT_ROI.origX) + "&y=" + str(self.CURRENT_ROI.origY) +
            "&z=" + str(self.CURRENT_ROI.origZ) + "&w=" + 
            str(self.CURRENT_ROI.sizeX) + "&h=" + str(self.CURRENT_ROI.sizeY) +
            "&d=" + str(self.CURRENT_ROI.sizeZ))
    print(req)
    ret = requests.get(req)
    processDone = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        processDone = True
        print("BBox updated.")
    return processDone
  
  def initApp(self, username, model, mdlPublic, t1, t1Public, force_retry):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/init")
    headers = {'Content-Type': 'application/json', 'Accept':'application/json'}
    dt = { "model": model, "modelPublic": str(mdlPublic), "inputT1Image": t1, "t1Public": str(t1Public), "retry": force_retry }
    print(dt)
    ret = requests.post(req, data = json.dumps(dt), headers = headers)
    self.initialized = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        self.initialized=True
  
  def initSegmentationMethod(self, method):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/segmethod/?m=" + method)
    print(req)
    ret = requests.get(req)
    initialized = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        initialized=True

    return initialized
  
  #deprecated
  def getCurrentImageURL(self, image, ispublic):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
         "/neurobrainweb/data/image/" + image + '?userID=' + self.baseLogic.USER_ID + 
         "&isPublic=" + str(ispublic))
    print(req)
    ret = requests.get(req)
    imgURL = ""
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      imgURL = (self.baseLogic.SERVER_ADRS + ":" + 
                self.baseLogic.SERVER_PORT + resp["img_path"])
    print(imgURL)
    # imgURL = (self.baseLogic.SERVER_ADRS + ":" +
        # self.baseLogic.SERVER_PORT + "/Data/Images/" + image)
    return imgURL
    
  def listRegions(self):
    regionDic = {}
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/regions/")
    ret = requests.get(req)
    initialized = False
    print(req, ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        #print(resp["data"]["regions"])
        regions = resp["data"]["regions"]
        
        #print "regions="+regions
        for region in regions:
          regionDic[int(region["id"])] = region["name"]
        
        regionDicOrd = collections.OrderedDict(sorted(regionDic.items()))
        # print(regionDic)
        # print(regionDicOrd)

    return regionDicOrd
    
  def removeSegmentedRegion(self, region_id):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/region/remove/?id=" + str(region_id))
    print(req)
    ret = requests.get(req)
    regionIDremoved = False
    print(ret)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        regionIDremoved = True

    return regionIDremoved

  def startSegmentationNew(self):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/segmentation/start")
    
    ret = requests.get(req)
    started = False
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        started = True
    else:
      [msg, retry] = self.baseLogic.hanldeFatalError(ret)
      self.instanceModuleGUI.delayDisplay(msg, 4000)
      if retry:
        self.baseLogic.FORCE_RETRY = True
        # some global variables to reset
        self.SEG_TIMER = None
        self.SEG_CNT_TIMER_RAN = 0
        self.baseLogic.forceModuleReload("SegModule")
        self.baseLogic.forceModuleReload("ModelModule")
        self.baseLogic.switchToTab(slicer, "SegModule")
    
    return started

  def checkSegmentationStatus(self):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/process/check/segmentation")
    
    ret = requests.get(req)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        if resp["done"]:
          return [True, True, resp["message"]]
        else:
          return [True, False, resp["message"]]
      else:
        if resp["done"] is not None and resp["done"]:
          return [False, True, resp["message"]]
        else:
          return [False, False, resp["message"]]
    else:
      print(ret.json())
      [msg, retry] = self.baseLogic.hanldeFatalError(ret)
      self.instanceModuleGUI.delayDisplay(msg, 4000)
      if retry:
        self.baseLogic.FORCE_RETRY = True
        # some global variables to reset
        self.SEG_TIMER = None
        self.SEG_CNT_TIMER_RAN = 0
        self.baseLogic.forceModuleReload("SegModule")
        self.baseLogic.forceModuleReload("ModelModule")
        self.baseLogic.switchToTab(slicer, "SegModule")
      return [False, False, "Unexpected error has occured!"]
          
  def unloadSegmentation(self):
    req = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT +
            "/neurobrainweb/" + self.baseLogic.USER_ID + 
            "/segmentation/unload")
    
    ret = requests.get(req)
    if ret.status_code == 200:
      resp = ret.json()
      print(resp)
      if resp["status"] == "OK":
        return True
        
    return False
  
  def uploadSaveCustomSession(self):
    # save the the labelmap to a tmp file to upload it later
    name = "current_results.nrrd"
    file_output_nrrd = slicer.app.temporaryPath + '/' + name
    if os.path.exists(file_output_nrrd):
      os.remove(file_output_nrrd)
    myStorageNode = self.segmentationVolumeNode.CreateDefaultStorageNode()
    myStorageNode.SetFileName(file_output_nrrd)
    myStorageNode.WriteData(self.segmentationVolumeNode)
    
    # -------------------- START Try to convert nrrd to nifti -------------------------------
    try:
      file_input_irm = slicer.app.temporaryPath + '/' + self.baseLogic.CURRENT_T1_FILE
      if self.baseLogic.LOADED_SESSION_DIRNAME != "":
        file_input_ori = slicer.app.temporaryPath + '/' + 'results.nii.gz'
      else:
        file_input_ori = ''
      file_output_nii = slicer.app.temporaryPath + '/' + 'current_results.nii.gz'
      
      #load irm img
      nib.Nifti1Header.quaternion_threshold = -1e-06 # to solve issue https://github.com/nipy/nibabel/issues/626
      img_irm = nib.load(file_input_irm)
      #print(img_irm.shape)
      
      #load nrrd img
      img_nrrd = nrrd.read(file_output_nrrd)
      data_nrrd = img_nrrd[0]
      header_nrrd = img_nrrd[1]
      #print(header_nrrd)
      #get header of existing result
      if file_input_ori != '':
        img_ori = nib.load(file_input_ori)
        #print(img_ori.header)
      else:
        img_ori = None
        
      #save nifti
      dim_nrrd = int(header_nrrd['dimension'])
      if dim_nrrd == 3: # only one region in the segmentation
          regions = 1
          data_nii = data_nrrd[:, :, :] * 0
          data_nii = (data_nii + data_nrrd[:, :, :] *
                      int(header_nrrd['Segment0_Name']))
      else:
          regions = data_nrrd.shape[0]
          data_nii = data_nrrd[0, :, :, :] * 0
          for r in range(regions):
              data_nii = (data_nii + (data_nrrd[r, :, :, :] * 
                                      int(header_nrrd['Segment' + str(r) + '_Name'])))
      #enlarge image to origin size
      _BB = header_nrrd['space origin']
      ori_BB = img_irm.affine
      inverseORI = np.linalg.inv(ori_BB)
      vectorORI_1 = np.ones(4)
      vectorORI_2 = np.ones(4)
      vectorORI_1[0:3] = ori_BB[0:3,3]
      vectorORI_2[0:3] = _BB
      offset =  np.dot(inverseORI, vectorORI_2) - np.dot(inverseORI, vectorORI_1)
      offset  = np.round(offset)
      #offset_x = (_BB[0] - ori_BB[0,3])/ori_BB[0,0]
      #offset_y = (_BB[1] - ori_BB[1,3])/ori_BB[1,1]
      #offset_z = (_BB[2] - ori_BB[2,3])/ori_BB[2,2]
      offset_x = int(round(offset[0]))
      offset_y = int(round(offset[1]))
      offset_z = int(round(offset[2]))
      
      data_nii_fin = img_irm.get_fdata() * 0
      data_nii_fin[offset_x:offset_x + data_nii.shape[0],
                   offset_y:offset_y + data_nii.shape[1],
                   offset_z:offset_z + data_nii.shape[2]] = data_nii
      if img_ori != None:
        img_nii = nib.Nifti1Image(data_nii_fin, img_ori.affine, img_ori.header)
      else:
        img_nii = nib.Nifti1Image(data_nii_fin, img_irm.affine, img_irm.header)
      #print(img_nii.header)
      nib.save(img_nii, file_output_nii)
    except:
      self.instanceModuleGUI.delayDisplay('Wrong labelmap format.\n' + 
                        'Possible reason: region names are not numbers in segmentation editor.\n'+
                        'Please retry again and if problem persists, contact SILA3D support team.', 3000)
      return False
    # -------------------- END Try to convert nrrd to nifti -------------------------------
    
    # localLabelmap = slicer.mrmlScene.AddNewNodeByClass('vtkMRMLLabelMapVolumeNode')
    # slicer.modules.segmentations.logic().ExportVisibleSegmentsToLabelmapNode(self.segmentationVolumeNode, localLabelmap, self.labelmapVolumeNode)
    # slicer.modules.segmentations.logic().ExportAllSegmentsToLabelmapNode(self.segmentationVolumeNode, localLabelmap)
    # slicer.util.saveNode(localLabelmap, filePath)
    # print(self.segmentationVolumeNode.GetBinaryLabelmapRepresentation("3"))
    # upload it to server
    url = (self.baseLogic.SERVER_ADRS + ":" + self.baseLogic.SERVER_PORT + 
            "/neurobrainweb/" + self.baseLogic.USER_ID + "/" +
            self.baseLogic.CURRENT_MODEL + "/customSession")
    
    session = requests.Session()
    with open(file_output_nii, 'rb') as f:
      form = encoder.MultipartEncoder({
        "file": (os.path.basename(file_output_nii), f, "application/octet-stream")
      })
      headers = {"Prefer": "respond-async", "Content-Type": form.content_type}
      response = session.post(url, headers=headers, data=form)
    session.close()
    print(str(response.status_code) + " :: " + response.content)
    ret = response.json()
    if ret["status"] == "OK":
      return True
    else:
      return False

class SegModuleTest(ScriptedLoadableModuleTest):
  """
  This is the test case for your scripted module.
  Uses ScriptedLoadableModuleTest base class, available at:
  https://github.com/Slicer/Slicer/blob/master/Base/Python/slicer/ScriptedLoadableModule.py
  """

  def setUp(self):
    """ Do whatever is needed to reset the state - typically a scene clear will be enough.
    """
    slicer.mrmlScene.Clear(0)

  def runTest(self):
    """Run as few or as many tests as needed here.
    """
    self.setUp()
    self.test_TestModuleTest1()

  def test_TestModuleTest1(self):
    """ Ideally you should have several levels of tests.  At the lowest level
    tests should exercise the functionality of the logic with different inputs
    (both valid and invalid).  At higher levels your tests should emulate the
    way the user would interact with your code and confirm that it still works
    the way you intended.
    One of the most important features of the tests is that it should alert other
    developers when their changes will have an impact on the behavior of your
    module.  For example, if a developer removes a feature that you depend on,
    your test should break so they know that the feature is needed.
    """

    pass
